//============================================================================
// FILE: Descriptor.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe des descripteurs
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _DESCRIPTOR_HH
#define _DESCRIPTOR_HH

// SYSTEM INCLUDES
//
//#include <>                                    //

// PROJECT INCLUDES
//
#include "CoreTypes.h"
#include "DescriptedEntity.h"

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name CDPFORTEXAttributs \`a valeur explicite des entit\'es : les descripteurs
Le système simulé est composé d'entités entre lesquelles existent des relations structurales (composition, inclusion), gérées par des services de la classe [C]Entity[/C]. Les descripteurs permettent quant à eux de spécifier deux autres types d'information sur une entité :
\begin{itemize}
\item Les caractéristiques propres d'une entité, qui correspondent aux notions de {\em variables d'état}\index{variables d'état} (du système simulé) et de {\em constantes}\index{constantes}. Une constante est simplement une variable dont la valeur initiale est censée de plus changer au cours de la simulation. 
\item Les relations non structurales liant l'entité, vu comme un {\em antécédent}, à une {\em image} selon une sémantique quelconque\index{relation antécédent/image}. La relation {\em père}/{\em fils} en est un exemple. Comme la relation est en général réciproque, deux descripteurs différents sont attachés aux deux entités liés.
\end{itemize}

Dans DIESE, la valeur d'un descripteur peut être un entier ou un réel en simple ou double précision (resp. les types {\em int}, {\em float} et {\em double} de C), une chaîne de caractères (le type {\em char*} de C). Cela peut aussi être un tableau de chaînes de caractères, une instance d'[C]Entity[/C], un tableau de pointeurs sur des instances d'[C]Entity[/C], une instance d'[C]EntitySpec[/C], un tableau de pointeurs sur des instances d'[C]EntitySpec[/C], un tableau d'intervalles (entiers ou réels) ou enfin un tableau de réels en simple précision. C'est en cela qu'on dit que la valeur d'un descripteur est explicite, au contraire des attributs fonctionnels (voir la classe [C]Method[/C] qui définissent une valeur calculée dynamiquement par une procédure.\\

Les descripteurs des entités ne peuvent être des descendants que des deux sous-classes prédéfinies de [C]Descriptor[/C] : [C]ConstantDescriptor[/C] et [C]VariableDescriptor[/C]. Seuls les services applicables à tous les types de descripteurs sont définis au niveau de la présente classe de base. En particulier, les services renvoyant un type prédéfini de C ou une instance (ou un tableau) d'[C]Entity[/C] (ou qui en utilisent en argument) sont décrits dans les sous-classes correspondant à ces types.\\

Les descripteurs scalaires numériques sont regroupés dans deux sous-classes [C]MonoDimensionalNumerical[Constant Variable]Descriptor[/C]. Ce sont ceux dont le type de valeur est {\em int}, {\em float}, {\em double}, {\em pIntIntervalTab*}, {\em pFloatIntervalTab*}, {\em pDoubleIntervalTab*}. Ils possèdent un attribut caractéristique : l'unité de la valeur scalaire, accessible par les services [Y]ClassUnity(char*)[/Y] et [Y]char* ClassUnity()[/Y].

Qu'il soit constant ou variable, un descripteur peut être doté, normalement lors de sa construction, d'un domaine de valeurs admissibles\index{domaine de valeurs -Descriptor-}, par les services [Y]SetSubDomain[/Y] et [Y]AddSubDomain[/Y] des deux sous-classes. Une exception sera alors levée (telle que [Y]OutOfIntDomain[/Y], \ldots) si on tente d'affecter une valeur non admissible, par [Y]SetValue[/Y] adressé au descripteur ou par [Y]Set..Value[/Y] adressé à l'entité. 
\noLF
Attention! Ce ne sont pas les constructeurs des classes prédéfinies qui dotent le descripteur d'un domaine de valeurs admissibles (même infiniment grand pour les descripteurs numériques). C'est dire qu'aucune exception ne sera levée, quelle que soit la valeur affectée, si un service tel que [Y]SetSubDomain[/Y] n'est pas utilisé dans les constructeurs des classes propres à l'application.
*/
//@{  // fermeture dans CoreVariableDescriptor.h

/**@name La classe de base pour tous les descripteurs.
\begin{itemize} \item Directive d'inclusion :
[P]
\#include Descriptor.h
[/P]
\end{itemize}
\label{defDescriptor}
*/
//@{
        ///
class Descriptor 
{
public:

// @Man: LIFECYCLE
// @{
          // Constructeur de base. N'est normalement appelé que par les sous-classes prédéfinies, et non dans les classes définies par l'utilisateur.
   Descriptor();  // _no swig_    
          // Destructeur.
   ~Descriptor();   // _no swig_  
// @}

// @Man: OPERATORS
// @{
         // Opération  d'affectation. Cette opération n'est en fait pas gérée par DIESE : l'exception [Y]UnAuthorizedAssignement[/Y] est levée.

   const Descriptor&     operator=(const Descriptor&);  // assignment operator
// @}

// OPERATIONS
                  

/**@name [SC]Identification[/SC] */
//@{
         /// Affecte une catégorie au descripteur qui reçoit ce message.\index{méthodes!Subclass - Descriptor}
   void              Subclass(DescriptorSubclassId);  // _no swig_
         /// Renvoie la catégorie du récepteur.
   DescriptorSubclassId Subclass() const;  // _no swig_
         /// Affecte un type de valeur au descripteur qui reçoit ce message.\index{méthodes!ClassType - Descriptor}
   void              ClassType(ValueTypeId);  // _no swig_
         /// Renvoie le type de valeur du récepteur.
   ValueTypeId       ClassType() const;  // _no swig_
         /// Affecte un symbole de classe au récepteur.\index{méthodes!ClassSymbol - Descriptor}
   void              ClassSymbol(UserDescriptorId);  // _no swig_
         /// Renvoie le symbole de classe du récepteur.
   UserDescriptorId  ClassSymbol() const;  // _no swig_
         /// Affecte un nom de classe au descripteur qui reçoit ce message. Pour une exploitation dans le fichier de spécifications de sorties, le nom ne doit comporter que des caractères ASCII\index{nom de classe (descripteur)}.\index{méthodes!ClassName - Descriptor}
   void              ClassName(char*);  // _no swig_
         /// Renvoie le nom de classe du récepteur.
   char*             ClassName() const;  // _no swig_
         /// Affecte un commentaire au descripteur qui reçoit ce message.\index{méthodes!ClassComment - Descriptor}
   void              ClassComment(char*);  // _no swig_
         /// Renvoie le commentaire associé au récepteur.
   char*             ClassComment() const;  // _no swig_

         /// Renvoie {\em vrai} si le descripteur est constant, et {\em faux} sinon.\index{méthodes!IsConstant}
   Boolean           IsConstant() const;  // _no swig_

         /// Renvoie {\em vrai} si le descripteur est variable, et {\em faux} sinon.\index{méthodes!IsVariable}
   Boolean           IsVariable() const;  // _no swig_

         /// Renvoie {\em vrai} si le descripteur est entier, et {\em faux} sinon.\index{méthodes!IsInteger}
   Boolean           IsInteger() const;  // _no swig_

         /// Renvoie {\em vrai} si le descripteur est flottant simple précision, et {\em faux} sinon.\index{méthodes!IsFloat}
   Boolean           IsFloat() const;  // _no swig_

         /// Renvoie {\em vrai} si le descripteur est flottant double précision, et {\em faux} sinon.\index{méthodes!IsDouble}
   Boolean           IsDouble() const;  // _no swig_ 

  // début code gestion OutputSpec
          /// Détermine si la valeur du descripteur peut être enregistrée (valeur {\em TRUE}) ou non (valeur {\em FALSE}) au titre des spécifications de sortie, si une a été énoncée. La valeur {\em TRUE} a été passée à ce service quand le descripteur a fait l'objet d'une spécification de sortie. Ce service peut être utilisé pour débrayer momentanément (puis rembrayer) l'enregistrement de valeurs.
   void              IsRecorded(Boolean);   // _no swig_ 

          /// Dans le cas où le descripteur a fait l'objet d'une spécification de sortie non, débraye le mécanisme d'enregistrement des changements de valeur s'il n'est pas déjà débrayé.
   void              DisableRecording();
          /// Dans le cas où le descripteur a fait l'objet d'une spécification de sortie non, embraye le mécanisme d'enregistrement des changements de valeur s'il est débrayé. 
   void              EnableRecording();
          // Renvoie TRUE s'il a été spécifié que la valeur du descripteur pouvait être enregistrée au titre des spécifications de sortie.\index{méthodes!Descriptor - IsRecorded}
   Boolean           IsRecorded() const;   // _no swig_ 
   // fin code gestion OutputSpec 
 
//@}   

/**@name [SC]Entité décrite[/SC]
\\

La première de ces méthodes est évoquée pour information. Elle est exploitée par le moteur lors de l'attachement d'un descripteur à une entité (méthode [Y]Add[Constant Variable]Descriptor[/Y]). Le développeur d'une application n'a normalement pas de motif pour utiliser ce service.
*/
//@{

         /// Etablit que l'entité en argument est celle que décrit le récepteur.\index{méthodes!DescribedEntity}
   void              DescribedEntity(DescriptedEntity*);  // _no swig_
         /// Renvoie l'entité que décrit le récepteur.
   DescriptedEntity* DescribedDescriptedEntity() const;
         /// Renvoie l'entité que décrit le récepteur. 
   Entity*           DescribedEntity() const;
//@}


   //--------------- addition JPR 050909 pour WriteToCls
         // Renvoie {\em vrai} si le descripteur n'est exploité que par le moteur, et {\em faux} sinon.
   Boolean           IsPrivate() const;  // _no swig_
         //
   void              SetIsPrivate(Boolean);  // _no swig_

  // début code gestion OutputSpec
          // Etablit que c'est la structure passée en argument qui contient la valeur finale du descripteur.
   void                 SetDescTimeSeries(DescTimeSeries*);   // _no swig_
          // Renvoie la structure qui contient la valeur finale du descripteur.
   DescTimeSeries*      GetDescTimeSeries() const;   // _no swig_
   // fin code gestion OutputSpec 

         //
   void              ValueToString(char str[50]);  // virtual : sup JPR 050407
         //
   void              PlotValue();  // _no swig_

         // comptage
    static int   getNbConst() {return _nbConst;}
    static int   getNbDest() {return _nbDest;} 

    static int   getNbConstC() {return _nbConstC;}
    static int   getNbDestC() {return _nbDestC;} 

    static int   getNbConstV() {return _nbConstV;}
    static int   getNbDestV() {return _nbDestV;} 
// INQUIRY

protected:

   int static   _nbConst;
   int static   _nbDest;
   int static   _nbConstC;
   int static   _nbDestC;
   int static   _nbConstV;
   int static   _nbDestV;

   DescriptorSubclassId  mDescriptorSubclass;
   ValueTypeId           mDescriptorType;
   UserDescriptorId      mDescriptorClassId;
   char*                 mDescriptorName;
   DescriptedEntity*     mpDescribedEntity;
   char*                 mDescriptorComment;
   
   //--------------- addition JPR 050909 pour WriteToCls
   Boolean               mIsPrivate; 

   // début code gestion OutputSpec // modif 071023 transfert de CD et VD
   Boolean              mIsRecorded;
   DescTimeSeries*      mDescTimeSeries;    
   // fincode gestion OutputSpec

private:
}; // _end class_

//@} // fin classe


//-X///////////////X/////////////////////////////X////////////////////////////

// INLINE METHODS

// EXTERNAL REFERENCES

#endif  // _DESCRIPTOR_HH


