//============================================================================
// FILE: Liste.h 
//
// TYPE: C++TYPEDEF
// DESCRIPTION: définition de la classe générique Liste
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _LISTE_H
#define _LISTE_H

// SYSTEM INCLUDES

// PROJECT INCLUDES

template <class T> class Liste;  // + JPR 990903

//----------------------------------------------------------------------------
template <class T> class Cell
{
public:
    Cell(T);
    Cell(T, Cell<T>*);                                  
          // Pas de constructeur par copie validé
    Cell(const Cell<T>&);
    ~Cell(); 

    T         GetContent();
    Cell<T>*  GetNextCell();
    void      SetContent(T);
    void      SetNextCell(Cell<T>*);
    void      display();
      
    friend class Liste<T>;
protected :
    T         contenu;
    Cell<T>*  nextCell;
}; // _end class_
//--------------------------------------------------------------------------
/**@name La classe générique des listes
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include "Liste.h"
[/P]
\end{itemize} 

Contrairement à la structure de [C]Tableau[/C], une [C]Liste[/C] est une structure de donnée séquentielle. Elle peut être vide ou bien comprendre un élément de tête, un élément de queue et d'autres éléments. Un élément est l'élément courant.\\

Une liste supporte l'ajout d'élément en tête et en queue, et l'insertion avant ou après l'élément courant. Les éléments ne sont pas indicés et l'accès à un élément par son rang est donc impossible. La liste est typiquement explorée séquentiellement à partir de son élément de tête jusqu'à son élément de queue. Voici une illustration de ce traitement :
\begin{verbatim}
[P]int main (...) { 
  ... 
  Liste<int>* pList = ...; 
  pList->go_head();
  while(pList->is_current_in()) {
    int elem_k = pList->current();
    ... // traitement de elem_k
    pList->go_next();
  };
}[/P]
\end{verbatim}


*/
//@{

/**@name La classe : template <class T> class Liste */
//@{
          ///
template <class T> class Liste
{
public:

/**@name LIFECYCLE
\label{constDestListe}
Une trace\index{trace (création/destruction)} de la création et de la destruction des listes peut \^etre obtenue en donnant auparavant la valeur {\small {\tt TRACE_LISTE_ON}} à la variable [/Y]gTraceListeMode[/Y]\index{gTraceListeMode}. Sinon, la trace doit \^etre inhibée avec la valeur {\small {\tt TRACE_LISTE_OFF}}, quelque part dans le programme de l'utilisateur.
\begin{verbatim}
[P]int gNbConstListe =0;
int gNbDestListe =0; 
int gNbConstCell =0; 
int gNbDestCell =0;  
boolTraceListeMode gTraceListeMode =TRACE_LISTE_ON; 

int main (...) { 
  ... 
}[/P]
\end{verbatim}

Les constructeurs et le destructeur incrémentent et décrémentent le nombre de listes créées et détruites, respectivement dans les variables globales [Y]gNbConstListe[/Y]\index{gNbConstListe} et [Y]gNbDestListe[/Y]\index{gNbDestListe}. Ces deux variables doivent être initialisées à 0 quelque part dans le programme. Pour les utilisateurs de DIESE, cette initialisation est réalisée dans [F]CoreTypes.o[/F]}.
*/
//@{
          /// Constructeur de base d'une liste vide.
   Liste();
          // Constructeur d'une liste avec un élément de tête T. 
   Liste(T elem);                                  
          /// Constructeur par copie. Les éléments de la copie ont la même valeur que les éléments de l'original (affectation). Si ces éléments sont des pointeurs, la copie pointe sur les mêmes éléments que l'original. 
   Liste(const Liste<T>&);
          /// Destructeur.                
   ~Liste(void); 
//@}             

/**@name  OPERATORS
De façon générale, lorsque le prérequis d'un opérateur ou d'une opération n'est pas satisfait, le programme appelant est arr\^eté. L'adresse de la liste pour lequel l'opération a échoué est affichée. L'utilisateur peut utiliser la variable globale [Y]gTraceListeMode[/Y]\index{gTraceListeMode} pour avoir la trace de la création de la liste.
*/ 
//@{         
          // Affectation. 
      const Liste<T>&    operator = (const Liste<T>& source);

          /// Opérateur de test d'égalité. Les deux listes sont égales (et l'opérateur renvoie 1) si pour tous les couples de cellules de même rang, leurs contenus sont égaux. Si les contenus sont ds pointeurs, les deux cellules doivent pointer sur le même entité. Sinon l'opérateur renvoie 0.\index{Liste_m!operator!==}
      //int            Liste<T>::operator == (const Liste<T>& source);
      int            operator == (const Liste<T>& source); // gcc 4.1
//@}             

/**@name  ACCESS */
//@{    
          ///  Renvoie TRUE si la liste est vide, FALSE sinon.\index{Liste_m!access!empty} 
   int            empty();    
          //  Renvoie la premiere cellule de la liste.
Cell<T>*          get_headCell();       
          //  Etablit la premiere cellule de la liste, qui devient la cellule courante.
void              set_headCell(Cell<T>*);  
          //  Renvoie la cellule courante de la liste.
Cell<T>*          get_currentCell();  
          //  Etablit la cellule courante de la liste.
void              set_currentCell(Cell<T>*);  
          ///  Renvoie le premier élément de la liste. La liste ne doit pas \^etre vide.\index{Liste_m!access!front}    
   T              front() const;   
          ///  Renvoie le premier élément de la liste. La liste ne doit pas \^etre vide.\index{Liste_m!access!head}    
   T              head() const; 
          ///  Renvoie l'élément courant de la liste. La liste ne doit pas \^etre vide.\index{Liste_m!access!current}    
   T              current() const;
          ///  Renvoie {\em vrai} si l'élément courant est le dernier de la liste.\index{Liste_m!access!is_current_last}    
   bool           is_current_last() const; 
          ///  Renvoie {\em vrai} si l'élément courant est NULL, élément "virtuel" qui suit le dernier de la liste.\index{Liste_m!access!is_current_off}    
   bool           is_current_off() const; 
          ///  Renvoie {\em vrai} si l'élément courant est non NULL, c'est-à-dire un élément contenu dans la liste.\index{Liste_m!access!is_current_in}    
   bool           is_current_in() const;
         //
   void      display();
          // 
   void      displayContent();
//@}

/**@name  OPERATIONS */
//@{
          ///  Insère un élément en tête de la liste, même si elle est vide. Cet élément devient l'élément courant.\index{Liste_m!operation!push_front} 
   void            push_front(const T &);

          ///  Insère un élément après l'élément courant. L'élément inséré devient l'élément courant. Si la liste est vide, cette méthode a le même effet que [Y]push_front[/Y].\index{Liste_m!operation!insert_after_current} 
   void            insert_after_current(const T &);   
 
          ///  Le premier élément devient l'élément courant. Si la liste est vide l'élément courant devient l'élément NULL.\index{Liste_m!operation!go_head}
   void            go_head();

          ///  L'élément suivant l'élément courant devient le nouvel élément courant. La liste ne doit pas être vide.\index{Liste_m!access!go_next}
   void            go_next();

          //  L'élément suivant l'élément courant devient le nouvel élément courant. La liste ne doit pas être vide. Renvoie 'faux' si l'élément suivant n'a pas été trouvé, 'vrai' sinon. Une alerte est émise si msg est TRUE.
   bool            go_next2(bool msg);  // + JPR 090414 modifié 100212

          ///  Le dernier élément devient l'élément courant. Si la liste est vide l'élément courant devient l'élément NULL.\index{Liste_m!operation!go_last}
   void            go_last();
  
          ///  Ajoute un élément à la liste, après le dernier. L'élément ajouté devient l'élément courant. Si la liste est vide, cette méthode a le même effet que [Y]push_front[/Y].\index{Liste_m!operation!push_back(const T \&)}
   void            push_back(const T &); 

          ///  Retire le premier élément de la liste s'il en existe au moins un. Laisse la liste intacte sinon. L'élément courant devient l'ancien deuxième élément, s'il y en avait un.\index{Liste_m!operation!pop_front}
   void            pop_front(); 

          ///  Retire le premier élément de la liste. La liste ne doit pas \^etre vide. Le nouveau premier élément devient l'élément courant. Returne l'ancien premier élément.\index{Liste_m!operation!pop_return_front}
   T               pop_return_front();

          ///  Retire de la liste l'élément courant. La liste ne doit pas \^etre vide. L'élément suivant l'ancien élément courant devient le nouvel élément courant.\index{Liste_m!operation!remove_current}
   void            remove_current();

          ///  Retire de la liste la première occurrence de l'élément passé en argument. L'élément suivant cette occurrence devient l'élément courant. La liste peut \^etre vide. Renvoie 1 si le retrait a pu avoir lieu et 0 sinon.\index{Liste_m!operation!remove_first}
   //T            remove_first(const T&);
   bool            remove_first(const T&);    // JPR 0703.. gcc 4.1

          ///  Remplace l'élément courant par l'élément donné en argument. \index{Liste_m!operation!replace_current_by_item}
   void            replace_current_by_item(const T &);
          ///  Retire tous les éléments de la liste, si elle en a.\index{Liste_m!operation!erase}
   void            erase();
          ///  Renvoie le nombre d'éléments dans la liste. L'élément courant devient l'élément de tête.\index{Liste_m!operation!size}
   int                size();
//@}   

protected :    

   Cell<T>*         headCell;
   Cell<T>*         currentCell;
   Cell<T>*         tailCell;

}; // _end class_

//@} // fin classe  
//@} // fin Liste

//@} // fermeture dans Tableau.h

#endif  //_LISTE_H
