//============================================================================
// FILE: C_ActivityNonPrimitive.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des sous-classes d'activités non primitives (C-DIESE)
// 
// AUTHORS: Baali/Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_ACTIVITY_NONPRIMITIVE_HH
#define _C_ACTIVITY_NONPRIMITIVE_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "C_Activity.h"
#include "C_CoreTypes.h"

/**@name Les opérateurs de composition d'activités
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_ActivityNonPrimitive.h} \end{itemize}

\begin{enumerate}
\item {\bf Définition}

Un {\em plan nominal} définit un ensemble d'opérations élémentaires (structuré par des {\em opérateurs de composition}\index{opérateur de composition}) à effectuer sur le système. Ce sont les {\em instructions}\index{instruction}, instances de la classe {\bf PrimitiveActivity}, qui spécifient ces opérations élémentaires, en termes de {\em ressources}\index{ressource} nécessaires à leur exécution et d'effet sur l'état du système. Parmi toutes les sous-classes d'{\bf Activity}, seules les {\em instructions} contiennent cette information.\\
Les {\em opérateurs de composition} correspondent aus différentes sous-classes de {\bf NonPrimitiveActivity}.
\end{enumerate}
*/
//@{
//----------------------------------------------------------------------------
// NON_PRIMITIVE_ACTIVITY
//---------------------------------------------------------------------------
// introduit le 030805 pour que les PrimitiveActivity puissent avoir des composants
//---------------------------------------------------------------------------

/**@name La classe de base pour les activités non primitives
\begin{enumerate}
\item {\bf Définition}
Une activité non primitive est une activité qui possède des éléments. Ces éléments sont des activités en général (primitives ou non). Une activité non primitive est créée pour déclarer un lien entre les activités qui sont ses éléments. Selon le sens du lien qu'on veut déclarer, on crée une instance de l'une ou l'autre des différentes spécialisations d'activités non primitives.
}
 */
//@{
        ///
class NonPrimitiveActivity : public Activity // _no swig_
{
public:
  NonPrimitiveActivity();

  ~NonPrimitiveActivity();

int  PropagateMinBegDateToSon(Activity* pSon);  // _no swig_
           //
int  PropagateMaxBegDateToSon(Activity* pSon);  // _no swig_
           //
int  PropagateMinEndDateToSon(Activity* pSon);  // _no swig_
           //
int  PropagateMaxEndDateToSon(Activity* pSon);  // _no swig_

  /**@name Etablissement d'un jeu d'instructions possiblement inconsistant */
  //@{
  ///
  void ResetSumDisjunction();  // _no swig_
  ///
  void ResetProductDisjunction();  // _no swig_
  ///
  void ResetUnaryDisjunction();  // _no swig_
  //@}  // fin Etablissement  ...
};
//@}  // fin classe de base

//---------------------------------------------------------------------------
 
   
//----------------------------------------------------------------------------
// ACTIVITY COMPOSITION OPERATORS
//----------------------------------------------------------------------------
/**@name Les opérateurs de composition sur les activités */
//@{
//----------------------------------------------------------------------------
// Activity REQUIRED
//----------------------------------------------------------------------------
        ///
class ActivityRequired : public NonPrimitiveActivity  // _no swig_
{
public:
ActivityRequired();

//TableauInt* OpeningWindow();

int  GetMinBegDateFromDown();
int  GetMaxBegDateFromDown();
int  GetMinEndDateFromDown();
int  GetMaxEndDateFromDown();

int  PropagateMinBegDateToSon(Activity* pSon);
int  PropagateMaxBegDateToSon(Activity* pSon);
int  PropagateMinEndDateToSon(Activity* pSon);
int  PropagateMaxEndDateToSon(Activity* pSon);

bool CheckSonsIfWaiting();  // _no swig_
bool CheckSonsIfWaiting(Activity*);  // _no swig_
bool CheckSonsIfOpen();  // _no swig_
bool CheckSonsIfOpen(Activity*);  // _no swig_
bool CheckSonsIfClosed();  // _no swig_
bool CheckSonsIfClosed(Activity*);  // _no swig_

bool CheckIfSonWaiting(Activity*);  // _no swig_
bool CheckIfSonOpen(Activity*);  // _no swig_
bool CheckIfSonClosed(Activity*);  // _no swig_

void PropagateWaitingToSons(Activity*);  // _no swig_
void PropagateOpenToSons(Activity*);  // _no swig_
void PropagateClosedToSons(Activity*);  // _no swig_

void UpdateIfSonWaiting(Activity*);  // _no swig_
void UpdateIfSonOpen(Activity*);  // _no swig_
void UpdateIfSonClosed(Activity*);  // _no swig_
void UpdateIfSonNoLongerOpenable(Activity*);  // _no swig_

bool         IsOrMayBeUniqueLast(Activity*);
bool         HasOperationNotBegunAtMaxBegDate();
bool         HasOperationNotBegun();

protected :
};  // _end class_
//@}

//----------------------------------------------------------------------------
// Activity PRECEDENCE
//----------------------------------------------------------------------------

/**@name Séquence d'activités disjointes
\label{classeActivityPrecedence}\index{opérateur de composition!précédence}
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_Activity.h} \item opérateur de composition : {\em before}\end{itemize}

Deux classes permettent de spécifier que deux ou plusieurs activités doivent se succéder dans le temps, sans aucun recouvrement. En d'autres termes il y a un délai non nul entre la fermeture d'une composante (passage de OPEN à CLOSED) et l'ouverture de la suivante (passage de WAITING à OPEN). \\

La précédence des {\em instructions} implique la précédence de l'exécution des opérations qui les composent respectivement. \\

La séquence suivante traduit la contrainte {\em before}({\em a1}, {\em a2}, {\em a3})  \\
\tt{
\ldots \\
Activity* a1 = new MyPrimitiveActivity();
Activity* a2 = new MyActivityBefore();
Activity* a3 = new MyActivityInclusion();
Activity* A = new ActivityBefore(); \\
A->AddElement(a1); \\
A->AddElement(a2); \\
A->AddElement(a3); \\
\ldots
}\\

La classe [C]ActivityBeforeAny[/C] contraint ses éléments à se succéder sans recouvrement, mais n'impose pas l'ordre de leur succession. Celui sera établi dynamiquement en cours de simulation en fonction du contexte courant.\\

{\bf descripteurs communs :} \\

{\scriptsize 
\noindent\begin{tabular}{lllll}
OPEN\_TO\_OPEN\_DELAYS  & openToOpenDelays  & {\bf pIntIntervalTab*} & NULL  \\
CLOSE\_TO\_OPEN\_DELAYS & closeToOpenDelays & {\bf pIntIntervalTab*} & NULL  \\
\end{tabular}
}\\
OPEN\_TO\_OPEN\_DELAYS : délai open->open
CLOSE\_TO\_OPEN\_DELAYS : délai closed->open

{\bf méthodes communes:} \ \\

{\scriptsize 
\noindent\begin{tabular}{lll}
\end{tabular} 
}
*/
//@{
          ///
//----------------------------------------------------------------------------
class ActivityBefore : public NonPrimitiveActivity  // _no swig_
{
public:
ActivityBefore();

//TableauInt* OpeningWindow();

int  GetMinBegDateFromDown();
int  GetMaxBegDateFromDown();
int  GetMinEndDateFromDown();
int  GetMaxEndDateFromDown();

int  PropagateMinBegDateToSon(Activity* pSon);
int  PropagateMaxBegDateToSon(Activity* pSon);
int  PropagateMinEndDateToSon(Activity* pSon);
int  PropagateMaxEndDateToSon(Activity* pSon);

bool CheckSonsIfWaiting();  // _no swig_
bool CheckSonsIfWaiting(Activity*);  // _no swig_
bool CheckSonsIfOpen();  // _no swig_
bool CheckSonsIfOpen(Activity*);  // _no swig_
bool CheckSonsIfClosed();  // _no swig_
bool CheckSonsIfClosed(Activity*);  // _no swig_

bool CheckIfSonWaiting(Activity*);  // _no swig_
bool CheckIfSonOpen(Activity*);  // _no swig_
bool CheckIfSonClosed(Activity*);  // _no swig_

void PropagateWaitingToSons(Activity*);  // _no swig_
void PropagateOpenToSons(Activity*);  // _no swig_
void PropagateClosedToSons(Activity*);  // _no swig_

void UpdateIfSonWaiting(Activity*);  // _no swig_
void UpdateIfSonOpen(Activity*);  // _no swig_
void UpdateIfSonClosed(Activity*);  // _no swig_
void UpdateIfSonNoLongerOpenable(Activity*);  // _no swig_

bool         IsOrMayBeUniqueLast(Activity*);
bool         HasOperationNotBegunAtMaxBegDate();
bool         HasOperationNotBegun();

protected :

};   // _end class_
          ///
//----------------------------------------------------------------------------
class ActivityBeforeAny : public NonPrimitiveActivity  // _no swig_
{
public:
ActivityBeforeAny();

//TableauInt* OpeningWindow();

void AddOperand(Activity*);
Activity* RemoveOperand(Activity*);
int GetNumberOfOperands();
Activity* GetOperand(int k);
Activity* GetOperand(); 
Activity* GetLastOperand();
int GetOperandPosition(Activity*);

int  GetMinBegDateFromDown();
int  GetMaxBegDateFromDown();
int  GetMinEndDateFromDown();
int  GetMaxEndDateFromDown();

int  PropagateMinBegDateToSon(Activity* pSon);
int  PropagateMaxBegDateToSon(Activity* pSon);
int  PropagateMinEndDateToSon(Activity* pSon);
int  PropagateMaxEndDateToSon(Activity* pSon);

bool CheckSonsIfWaiting();  // _no swig_
bool CheckSonsIfWaiting(Activity*);  // _no swig_
bool CheckSonsIfOpen();  // _no swig_
bool CheckSonsIfOpen(Activity*);  // _no swig_
bool CheckSonsIfClosed();  // _no swig_
bool CheckSonsIfClosed(Activity*);  // _no swig_

bool CheckIfSonWaiting(Activity*);  // _no swig_
bool CheckIfSonOpen(Activity*);  // _no swig_
bool CheckIfSonClosed(Activity*);  // _no swig_

void PropagateWaitingToSons(Activity*);  // _no swig_
void PropagateOpenToSons(Activity*);  // _no swig_
void PropagateClosedToSons(Activity*);  // _no swig_

void UpdateIfSonWaiting(Activity*);  // _no swig_
void UpdateIfSonOpen(Activity*);  // _no swig_
void UpdateIfSonClosed(Activity*);  // _no swig_
void UpdateIfSonNoLongerOpenable(Activity*);  // _no swig_

bool         IsOrMayBeUniqueLast(Activity*);
bool         HasOperationNotBegunAtMaxBegDate();
bool         HasOperationNotBegun();

protected :

};   // _end class_
 
//@}  // fin before


//----------------------------------------------------------------------------
// Activity MEETING
//----------------------------------------------------------------------------
/**@name Séquence d'activités jointes
\label{classeActivityMeeting}\index{opérateur de composition!jonction}
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_Activity.h} \item opérateur de composition : {\em meet} \end{itemize}

Les deux classes [C]ActivityMeeting[/C] et [C]ActivityMeetingAny[/C] permettent de spécifier que deux ou plusieurs activités doivent se succéder dans le temps sans aucun délai entre la fermeture d'une composante (passage de OPEN à CLOSED) et l'ouverture de la suivante (passage de WAITING à OPEN). \\

A la différence de [C]ActivityMeeting[/C], [C]ActivityMeetingAny[/C] n'impose pas l'ordre de leur succession. Celui sera établi dynamiquement en cours de simulation en fonction du contexte courant.\\
*/
//@{
          ///
//----------------------------------------------------------------------------
class ActivityMeeting : public NonPrimitiveActivity  // _no swig_
{
public:
ActivityMeeting();

//TableauInt* OpeningWindow();

int  GetMinBegDateFromDown();
int  GetMaxBegDateFromDown();
int  GetMinEndDateFromDown();
int  GetMaxEndDateFromDown();

int  PropagateMinBegDateToSon(Activity* pSon);
int  PropagateMaxBegDateToSon(Activity* pSon);
int  PropagateMinEndDateToSon(Activity* pSon);
int  PropagateMaxEndDateToSon(Activity* pSon);

bool CheckSonsIfWaiting();  // _no swig_
bool CheckSonsIfWaiting(Activity*);  // _no swig_
bool CheckSonsIfOpen();  // _no swig_
bool CheckSonsIfOpen(Activity*);  // _no swig_
bool CheckSonsIfClosed();  // _no swig_
bool CheckSonsIfClosed(Activity*);  // _no swig_

bool CheckIfSonWaiting(Activity*);  // _no swig_
bool CheckIfSonOpen(Activity*);  // _no swig_
bool CheckIfSonClosed(Activity*);  // _no swig_

void PropagateWaitingToSons(Activity*);  // _no swig_
void PropagateOpenToSons(Activity*);  // _no swig_
void PropagateClosedToSons(Activity*);  // _no swig_

void UpdateIfSonWaiting(Activity*);  // _no swig_
void UpdateIfSonOpen(Activity*);  // _no swig_
void UpdateIfSonClosed(Activity*);  // _no swig_
void UpdateIfSonNoLongerOpenable(Activity*);  // _no swig_

bool         IsOrMayBeUniqueLast(Activity*);
bool         HasOperationNotBegunAtMaxBegDate();
bool         HasOperationNotBegun();

protected :

};   // _end class_
          ///
//----------------------------------------------------------------------------
class ActivityMeetingAny : public NonPrimitiveActivity  // _no swig_
{
public:
ActivityMeetingAny();

//TableauInt* OpeningWindow();

void AddOperand(Activity*);
Activity* RemoveOperand(Activity*);
int GetNumberOfOperands();
Activity* GetOperand(int k);
Activity* GetOperand();  
Activity* GetLastOperand();
int GetOperandPosition(Activity*);

int  GetMinBegDateFromDown();
int  GetMaxBegDateFromDown();
int  GetMinEndDateFromDown();
int  GetMaxEndDateFromDown();

int  PropagateMinBegDateToSon(Activity* pSon);
int  PropagateMaxBegDateToSon(Activity* pSon);
int  PropagateMinEndDateToSon(Activity* pSon);
int  PropagateMaxEndDateToSon(Activity* pSon);

bool CheckSonsIfWaiting();  // _no swig_
bool CheckSonsIfWaiting(Activity*);  // _no swig_
bool CheckSonsIfOpen();  // _no swig_
bool CheckSonsIfOpen(Activity*);  // _no swig_
bool CheckSonsIfClosed();  // _no swig_
bool CheckSonsIfClosed(Activity*);  // _no swig_

bool CheckIfSonWaiting(Activity*);  // _no swig_
bool CheckIfSonOpen(Activity*);  // _no swig_
bool CheckIfSonClosed(Activity*);  // _no swig_

void PropagateWaitingToSons(Activity*);  // _no swig_
void PropagateOpenToSons(Activity*);  // _no swig_
void PropagateClosedToSons(Activity*);  // _no swig_

void UpdateIfSonWaiting(Activity*);  // _no swig_
void UpdateIfSonOpen(Activity*);  // _no swig_
void UpdateIfSonClosed(Activity*);  // _no swig_
void UpdateIfSonNoLongerOpenable(Activity*);  // _no swig_

bool         IsOrMayBeUniqueLast(Activity*);
bool         HasOperationNotBegunAtMaxBegDate();
bool         HasOperationNotBegun();

protected :

};   // _end class_
 
//@}  // fin meeting

//----------------------------------------------------------------------------
// Activity OVERLAPPING
//----------------------------------------------------------------------------
/**@name Recouvrement partiel d'activités
\label{classeActivityOverlapping}\index{opérateur de composition!recouvrement partiel}
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_Activity.h} \item opérateur de composition : {\em overlap} \end{itemize}

Cette classe permet de spécifier que deux ou plusieurs activités doivent se recouvrir partiellement dans le temps. En d'autres termes il y a un délai non nul entre l'ouverture d'une composante (passage de WAITING à OPEN) et l'ouverture de la suivante, un délai non nul entre l'ouverture de l'une et la fermeture de la précédente (passage de OPEN à CLOSED), et enfin un délai non nul entre la fermeture de l'une et la fermeture de la suivante. \\

{\bf descripteurs :} \\

{\scriptsize  
\noindent\begin{tabular}{lllll}
OPEN\_TO\_OPEN\_DELAYS  & openToOpenDelays   & {\bf pIntIntervalTab*} & NULL \\
OPEN\_TO\_CLOSE_DELAYS  & openToCloseDelays  & {\bf pIntIntervalTab*} & NULL \\
CLOSE\_TO\_CLOSE\_DELAYS & closeToCloseDelays & {\bf pIntIntervalTab*} & NULL \\
\end{tabular}
}\\
OPEN\_TO\_OPEN\_DELAYS : délai open->open
OPEN\_TO\_CLOSE\_DELAYS : délai open->closed
CLOSE\_TO\_CLOSE\_DELAYS : délai closed->closed

{\bf méthodes :} \\

{\scriptsize 
\noindent\begin{tabular}{lll}
\end{tabular} 
}

*/
//@{
        ///
//----------------------------------------------------------------------------
class ActivityOverlapping : public NonPrimitiveActivity  // _no swig_
{
public:
ActivityOverlapping();

//TableauInt* OpeningWindow();

int  GetMinBegDateFromDown();
int  GetMaxBegDateFromDown();
int  GetMinEndDateFromDown();
int  GetMaxEndDateFromDown();

int  PropagateMinBegDateToSon(Activity* pSon);
int  PropagateMaxBegDateToSon(Activity* pSon);
int  PropagateMinEndDateToSon(Activity* pSon);
int  PropagateMaxEndDateToSon(Activity* pSon);

bool CheckSonsIfWaiting();  // _no swig_
bool CheckSonsIfWaiting(Activity*);  // _no swig_
bool CheckSonsIfOpen();  // _no swig_
bool CheckSonsIfOpen(Activity*);  // _no swig_
bool CheckSonsIfClosed();  // _no swig_
bool CheckSonsIfClosed(Activity*);  // _no swig_

bool CheckIfSonWaiting(Activity*);  // _no swig_
bool CheckIfSonOpen(Activity*);  // _no swig_
bool CheckIfSonClosed(Activity*);  // _no swig_

void PropagateWaitingToSons(Activity*);  // _no swig_
void PropagateOpenToSons(Activity*);  // _no swig_
void PropagateClosedToSons(Activity*);  // _no swig_

void UpdateIfSonWaiting(Activity*);  // _no swig_
void UpdateIfSonOpen(Activity*);  // _no swig_
void UpdateIfSonClosed(Activity*);  // _no swig_
void UpdateIfSonNoLongerOpenable(Activity*);  // _no swig_

bool         IsOrMayBeUniqueLast(Activity*);
bool         HasOperationNotBegunAtMaxBegDate();
bool         HasOperationNotBegun();

protected :

};   // _end class_
 
//@}  // fin overlap

//----------------------------------------------------------------------------
// Activity INCLUSION
//----------------------------------------------------------------------------
/**@name Recouvrement total d'activités
\label{classeActivityInclusion}\index{opérateur de composition!recouvrement total}
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_Activity.h} \item opérateur de composition : {\em cover} \end{itemize}

Cette classe permet de spécifier que deux ou plusieurs activités doivent se recouvrir totalement dans le temps. En d'autres termes il y a un délai non nul entre l'ouverture d'une composante (passage de WAITING à OPEN) et l'ouverture de la suivante, et un délai non nul entre la fermeture de l'une et la fermeture de la précédente (passage de OPEN à CLOSED). \\

{\bf descripteurs :} \\

{\scriptsize 
\noindent\begin{tabular}{lllll}
OPEN\_TO\_OPEN\_DELAYS   & openToOpenDelays   & {\bf pIntIntervalTab*} & NULL \\
CLOSE\_TO\_CLOSE\_DELAYS & closeToCloseDelays & {\bf pIntIntervalTab*} & NULL \\
\end{tabular}
}\\
OPEN\_TO\_OPEN\_DELAYS : délai open->open
CLOSE\_TO\_CLOSE\_DELAYS : délai closed->closed

{\bf méthodes :}\\

{\scriptsize 
\noindent\begin{tabular}{lll}
\end{tabular} 
}

*/
//@{
          ///
//----------------------------------------------------------------------------
class ActivityInclusion : public NonPrimitiveActivity  // _no swig_
{
public:
ActivityInclusion();

//TableauInt* OpeningWindow();

int  GetMinBegDateFromDown();
int  GetMaxBegDateFromDown();
int  GetMinEndDateFromDown();
int  GetMaxEndDateFromDown();

int  PropagateMinBegDateToSon(Activity* pSon);
int  PropagateMaxBegDateToSon(Activity* pSon);
int  PropagateMinEndDateToSon(Activity* pSon);
int  PropagateMaxEndDateToSon(Activity* pSon);

bool CheckSonsIfWaiting();  // _no swig_
bool CheckSonsIfWaiting(Activity*);  // _no swig_
bool CheckSonsIfOpen();  // _no swig_
bool CheckSonsIfOpen(Activity*);  // _no swig_
bool CheckSonsIfClosed();  // _no swig_
bool CheckSonsIfClosed(Activity*);  // _no swig_

bool CheckIfSonWaiting(Activity*);  // _no swig_
bool CheckIfSonOpen(Activity*);  // _no swig_
bool CheckIfSonClosed(Activity*);  // _no swig_

void PropagateWaitingToSons(Activity*);  // _no swig_
void PropagateOpenToSons(Activity*);  // _no swig_
void PropagateClosedToSons(Activity*);  // _no swig_

void UpdateIfSonWaiting(Activity*);  // _no swig_
void UpdateIfSonOpen(Activity*);  // _no swig_
void UpdateIfSonClosed(Activity*);  // _no swig_
void UpdateIfSonNoLongerOpenable(Activity*);  // _no swig_

bool         IsOrMayBeUniqueLast(Activity*);
bool         HasOperationNotBegunAtMaxBegDate();
bool         HasOperationNotBegun();

protected :

};  // _end class_

//@}  // fin include

//----------------------------------------------------------------------------
// Activity COSTARTING
//----------------------------------------------------------------------------

/**@name Ouverture simultanée d'activités
\label{classeActivityCoStarting}\index{opérateur de composition!ouverture simultanée}
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_Activity.h} \item opérateur de composition : {\em co-start} \end{itemize}

Cette classe permet de spécifier que deux ou plusieurs activités doivent debuter en même temps : leurs dates d'ouverture (valeur du descripteur BEG\_DATE) doivent être égales. De plus, les {\em activités composantes} sont listées dans l'ordre croissant que doivent respecter leurs dates de fermeture (valeur du descripteur END\_DATE). \\

*/
//@{
          ///
//----------------------------------------------------------------------------
class ActivityCostarting : public NonPrimitiveActivity  // _no swig_
{
public:
ActivityCostarting();

//TableauInt* OpeningWindow();

int  GetMinBegDateFromDown();
int  GetMaxBegDateFromDown();
int  GetMinEndDateFromDown();
int  GetMaxEndDateFromDown();

int  PropagateMinBegDateToSon(Activity* pSon);
int  PropagateMaxBegDateToSon(Activity* pSon);
int  PropagateMinEndDateToSon(Activity* pSon);
int  PropagateMaxEndDateToSon(Activity* pSon);

bool CheckSonsIfWaiting();  // _no swig_
bool CheckSonsIfWaiting(Activity*);  // _no swig_
bool CheckSonsIfOpen();  // _no swig_
bool CheckSonsIfOpen(Activity*);  // _no swig_
bool CheckSonsIfClosed();  // _no swig_
bool CheckSonsIfClosed(Activity*);  // _no swig_

bool CheckIfSonWaiting(Activity*);  // _no swig_
bool CheckIfSonOpen(Activity*);  // _no swig_
bool CheckIfSonClosed(Activity*);  // _no swig_

void PropagateWaitingToSons(Activity*);  // _no swig_
void PropagateOpenToSons(Activity*);  // _no swig_
void PropagateClosedToSons(Activity*);  // _no swig_

void UpdateIfSonWaiting(Activity*);  // _no swig_
void UpdateIfSonOpen(Activity*);  // _no swig_
void UpdateIfSonClosed(Activity*);  // _no swig_
void UpdateIfSonNoLongerOpenable(Activity*);  // _no swig_

bool         IsOrMayBeUniqueLast(Activity*);
bool         HasOperationNotBegunAtMaxBegDate();
bool         HasOperationNotBegun();

protected :

};  // _end class_
//@}  // fin costart


//----------------------------------------------------------------------------
// Activity COENDING
//----------------------------------------------------------------------------
/**@name Fermeture simultanée d'activités
\label{classeActivityCoending}\index{opérateur de composition!fermeture simultanée}
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_Activity.h} \item opérateur de composition : {\em co-end} \end{itemize}

Cette classe permet de spécifier que deux ou plusieurs activités doivent finir en même temps : leurs dates de fermeture (valeur du descripteur END\_DATE) doivent être égales. De plus, les {\em activités composantes} sont listées dans l'ordre décroissant que doivent respecter leurs dates d'ouverture (valeur du descripteur BEG\_DATE). \\ 

Cet {\em opérateur de composition} appliqué à deux instructions n'entraîne pas le respect de l'ordre décroissant sur les débuts d'exécution des opérations les composant, dans le cas où l'exception {\bf ExecutionDelayed} est levée. \\


*/
//@{
          ///
//----------------------------------------------------------------------------
class ActivityCoending : public NonPrimitiveActivity  // _no swig_
{
public:
ActivityCoending();

//TableauInt* OpeningWindow();

int  GetMinBegDateFromDown();
int  GetMaxBegDateFromDown();
int  GetMinEndDateFromDown();
int  GetMaxEndDateFromDown();

int  PropagateMinBegDateToSon(Activity* pSon);
int  PropagateMaxBegDateToSon(Activity* pSon);
int  PropagateMinEndDateToSon(Activity* pSon);
int  PropagateMaxEndDateToSon(Activity* pSon);

bool CheckSonsIfWaiting();  // _no swig_
bool CheckSonsIfWaiting(Activity*);  // _no swig_
bool CheckSonsIfOpen();  // _no swig_
bool CheckSonsIfOpen(Activity*);  // _no swig_
bool CheckSonsIfClosed();  // _no swig_
bool CheckSonsIfClosed(Activity*);  // _no swig_

bool CheckIfSonWaiting(Activity*);  // _no swig_
bool CheckIfSonOpen(Activity*);  // _no swig_
bool CheckIfSonClosed(Activity*);  // _no swig_

void PropagateWaitingToSons(Activity*);  // _no swig_
void PropagateOpenToSons(Activity*);  // _no swig_
void PropagateClosedToSons(Activity*);  // _no swig_

void UpdateIfSonWaiting(Activity*);  // _no swig_
void UpdateIfSonOpen(Activity*);  // _no swig_
void UpdateIfSonClosed(Activity*);  // _no swig_
void UpdateIfSonNoLongerOpenable(Activity*);  // _no swig_

bool         IsOrMayBeUniqueLast(Activity*);
bool         HasOperationNotBegunAtMaxBegDate();
bool         HasOperationNotBegun();

protected :

};  // _end class_

//@}  // fin coend

//----------------------------------------------------------------------------
// Activity EQUALITY
//----------------------------------------------------------------------------
/**@name Concommitance d'activités
\label{classeActivityEquality}\index{opérateur de composition!concommitance}
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_Activity.h} \item opérateur de composition : {\em equal}\end{itemize}

Cette classe permet de spécifier que deux ou plusieurs {\em activités composantes} s'inscrivent dans la même période de temps. Pour toutes les classes d'activités (y compris les {\em instructions}, ce sont les valeurs des descripteurs BEG\_DATE et END\_DATE (c'est-à-dire les dates d'ouverture et de fermeture de l'activité, (cf. page \pageref{defBegDate}) qu'on force à être identiques dans toutes les {\em activités composantes}. \\

*/
//@{
        ///
//----------------------------------------------------------------------------
class ActivityEquality : public NonPrimitiveActivity  // _no swig_
{
public:
ActivityEquality();

//TableauInt* OpeningWindow();

int  GetMinBegDateFromDown();
int  GetMaxBegDateFromDown();
int  GetMinEndDateFromDown();
int  GetMaxEndDateFromDown();

int  PropagateMinBegDateToSon(Activity* pSon);
int  PropagateMaxBegDateToSon(Activity* pSon);
int  PropagateMinEndDateToSon(Activity* pSon);
int  PropagateMaxEndDateToSon(Activity* pSon);

bool CheckSonsIfWaiting();  // _no swig_
bool CheckSonsIfWaiting(Activity*);  // _no swig_
bool CheckSonsIfOpen();  // _no swig_
bool CheckSonsIfOpen(Activity*);  // _no swig_
bool CheckSonsIfClosed();  // _no swig_
bool CheckSonsIfClosed(Activity*);  // _no swig_

bool CheckIfSonWaiting(Activity*);  // _no swig_
bool CheckIfSonOpen(Activity*);  // _no swig_
bool CheckIfSonClosed(Activity*);  // _no swig_

void PropagateWaitingToSons(Activity*);  // _no swig_
void PropagateOpenToSons(Activity*);  // _no swig_
void PropagateClosedToSons(Activity*);  // _no swig_

void UpdateIfSonWaiting(Activity*);  // _no swig_
void UpdateIfSonOpen(Activity*);  // _no swig_
void UpdateIfSonClosed(Activity*);  // _no swig_
void UpdateIfSonNoLongerOpenable(Activity*);  // _no swig_

bool         IsOrMayBeUniqueLast(Activity*);
bool         HasOperationNotBegunAtMaxBegDate();
bool         HasOperationNotBegun();

protected :

};  // _end class_

//@}  // fin equal

//----------------------------------------------------------------------------
// Activity CONJUNCTION
//----------------------------------------------------------------------------
/**@name Conjonction d'opérateurs de composition
\label{classeActivityConjunction}\index{opérateur de composition!conjonction}
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_Activity.h} \item opérateur de composition : {\em and} \end{itemize}

Cette classe permet de spécifier que deux ou plusieurs contraintes doivent être conjointement satisfaites, autrement dit que deux ou plusieurs {\em opérateurs de composition} doivent être conjointement pris en compte. Noter qu'il n'y a pas d'intérêt à utiliser cet opérateur conjonctif avec une seule {\em activité composante}.\\

L'opérateur conjonctif sert essentiellement dans deux situations :
\begin{itemize}
\item Le {\em plan nominal} porte sur un ensemble d'activités partitionable, et des relations sont exprimées à l'intérieur de chaque partie mais pas entre activités de parties différentes. On aura alors une {\em activité quasi-racine} pour chaque partie, plus une conjonction d'activité avec les {\em activité quasi-racine} pour {\em composantes}.
\item L'ensemble des relations qu'on veut exprimer sur les activités du {\em plan nominal} est tel qu'aucun opérateur de composition, autre que {\em and}, ne peut constituer une {\em activité racine}. Soit, pour exemple, {\em a}, {\em b}, et {\em c}, trois activités. Les relations {\em meet(a, b)}, {\em co-end(b, c)} et {\em overlap(a, c)} ne peuvent être spécifiées au niveau du {\em plan nominal} que grâce à l'opérateur {\em and}, par exemple avec {\em and(overlap(a, co-end(b, c)), meet(a, b))}. Noter que dès lors que l'opérateur {\em and} est utilisé, une activité peut figurer dans deux de ses {\em activités composantes} (ici, {\em a} et {\em b}).
\end{itemize} 

*/
//@{
          ///
//----------------------------------------------------------------------------
class ActivityConjunction : public NonPrimitiveActivity  // _no swig_
{
public:
ActivityConjunction();

//TableauInt* OpeningWindow();

int  GetMinBegDateFromDown();
int  GetMaxBegDateFromDown();
int  GetMinEndDateFromDown();
int  GetMaxEndDateFromDown();

int  PropagateMinBegDateToSon(Activity* pSon);
int  PropagateMaxBegDateToSon(Activity* pSon);
int  PropagateMinEndDateToSon(Activity* pSon);
int  PropagateMaxEndDateToSon(Activity* pSon);

bool CheckSonsIfWaiting();  // _no swig_
bool CheckSonsIfWaiting(Activity*);  // _no swig_
bool CheckSonsIfOpen();  // _no swig_
bool CheckSonsIfOpen(Activity*);  // _no swig_
bool CheckSonsIfClosed();  // _no swig_
bool CheckSonsIfClosed(Activity*);  // _no swig_

bool CheckIfSonWaiting(Activity*);  // _no swig_
bool CheckIfSonOpen(Activity*);  // _no swig_
bool CheckIfSonClosed(Activity*);  // _no swig_

void PropagateWaitingToSons(Activity*);  // _no swig_
void PropagateOpenToSons(Activity*);  // _no swig_
void PropagateClosedToSons(Activity*);  // _no swig_

void UpdateIfSonWaiting(Activity*);  // _no swig_
void UpdateIfSonOpen(Activity*);  // _no swig_
void UpdateIfSonClosed(Activity*);  // _no swig_
void UpdateIfSonNoLongerOpenable(Activity*);  // _no swig_

bool         IsOrMayBeUniqueLast(Activity*);
bool         HasOperationNotBegunAtMaxBegDate();
bool         HasOperationNotBegun();

protected :

};  // _end class_

//@}  // fin and


//----------------------------------------------------------------------------
// Activity DISJUNCTION
//----------------------------------------------------------------------------
/**@name Activités alternatives
\label{classeActivityDisjunction}\index{opérateur de composition!disjonction}
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_Activity.h} \item opérateur de composition : {\em or} \end{itemize}

Cette classe permet de spécifier un choix entre deux ou plusieurs {\em activités alternatives}\index{activités alternatives}. Lorsqu'une est choisie et voit le début de son exécution, les autres sont annulées (le descripteur STATE passe à CANCELLED, c'est-à-dire que les opérations qu'elles induisent ne pourront plus être exécutées. \\
*/
//@{
          ///
//----------------------------------------------------------------------------
class ActivityDisjunction : public NonPrimitiveActivity  // _no swig_
{
public:
ActivityDisjunction();

//TableauInt* OpeningWindow();

int  GetMinBegDateFromDown();
int  GetMaxBegDateFromDown();
int  GetMinEndDateFromDown();
int  GetMaxEndDateFromDown();

int  PropagateMinBegDateToSon(Activity* pSon);
int  PropagateMaxBegDateToSon(Activity* pSon);
int  PropagateMinEndDateToSon(Activity* pSon);
int  PropagateMaxEndDateToSon(Activity* pSon);

bool CheckSonsIfWaiting();  // _no swig_
bool CheckSonsIfWaiting(Activity*);  // _no swig_
bool CheckSonsIfOpen();  // _no swig_
bool CheckSonsIfOpen(Activity*);  // _no swig_
bool CheckSonsIfClosed();  // _no swig_
bool CheckSonsIfClosed(Activity*);  // _no swig_

bool CheckIfSonWaiting(Activity*);  // _no swig_
bool CheckIfSonOpen(Activity*);  // _no swig_
bool CheckIfSonClosed(Activity*);  // _no swig_

void PropagateWaitingToSons(Activity*);  // _no swig_
void PropagateOpenToSons(Activity*);  // _no swig_
void PropagateClosedToSons(Activity*);  // _no swig_

void UpdateIfSonWaiting(Activity*);  // _no swig_
void UpdateIfSonOpen(Activity*);  // _no swig_
void UpdateIfSonClosed(Activity*);  // _no swig_
void UpdateIfSonNoLongerOpenable(Activity*);  // _no swig_

bool         IsOrMayBeUniqueLast(Activity*);
bool         HasOperationNotBegunAtMaxBegDate();
bool         HasOperationNotBegun();

protected :

};   // _end class_

//@}  // fin or

//---------------------------------------------------------------------------- 
// ACTIVITY_ITERATION
//----------------------------------------------------------------------------
/**@name Activités itératives
\label{classeActivityIteration}\index{opérateur de composition!itération}
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_Activity.h} \item opérateur de composition : {\em iterate} \end{itemize}

Cette classe permet d'exprimer qu'une activité (éventuellement non primitive) dite "itérée" doit être répétée à l'intérieur d'une période comprise entre l'ouverture et la fermeture de l'activité "itérative". \\

{\bf descripteurs :} \\

{\scriptsize 
\noindent\begin{tabular}{lllll}
OPEN\_TO\_OPEN\_DELAYS      & openToOpenDelays          & {\bf pIntIntervalTab*} & NULL \\
CLOSE\_TO\_OPEN\_DELAYS     & closeToOpenDelays         & {\bf pIntIntervalTab*} & NULL \\
MAXNUMBER\_OF\_REPLICATIONS & maxNumberOfReplications   & {\bf int} & INT\_MAX  \\
REPLICATION\_COUNTER        & replicationCounter        & {\bf int} & 0  \\
IS\_IN\_EXTRATIME           & isInExtratime & {\bf int} & faux(=0)  \\
\end{tabular}
}\\
OPEN\_TO\_OPEN\_DELAYS : délai open->open
CLOSE\_TO\_OPEN\_DELAYS : délai closed->open
MAXNUMBER\_OF\_REPLICATIONS : nb max d'itérations 
REPLICATION\_COUNTER : itérations déjà faites 
IS\_IN\_EXTRATIME : en cours après la fin 

{\bf méthodes :} \ \\

{\scriptsize 
\noindent\begin{tabular}{llll}
UPDATE_REACTIVATED_SON & updateReactivatedSon & spécifie la prochaine itération & {\bf VoidEntityMethod}  \\
\end{tabular} 
}

*/
//@{
          ///
//----------------------------------------------------------------------------
class ActivityIteration: public NonPrimitiveActivity  // _no swig_
{
public:
//ActivityIteration();

          /// Constructeur de base.
ActivityIteration();

//TableauInt* OpeningWindow();

int  GetMinBegDateFromDown();
int  GetMaxBegDateFromDown();
int  GetMinEndDateFromDown();
int  GetMaxEndDateFromDown();

int  PropagateMinBegDateToSon(Activity* pSon);
int  PropagateMaxBegDateToSon(Activity* pSon);
int  PropagateMinEndDateToSon(Activity* pSon);
int  PropagateMaxEndDateToSon(Activity* pSon);


bool CheckSonsIfWaiting();  // _no swig_
bool CheckSonsIfWaiting(Activity*);  // _no swig_
bool CheckSonsIfOpen();  // _no swig_
bool CheckSonsIfOpen(Activity*);  // _no swig_
bool CheckSonsIfClosed();  // _no swig_
bool CheckSonsIfClosed(Activity*);  // _no swig_

bool CheckIfSonWaiting(Activity*);  // _no swig_
bool CheckIfSonOpen(Activity*);  // _no swig_
bool CheckIfSonClosed(Activity*);  // _no swig_

void PropagateWaitingToSons(Activity*);  // _no swig_
void PropagateOpenToSons(Activity*);  // _no swig_
void PropagateClosedToSons(Activity*);  // _no swig_

void UpdateIfSonWaiting(Activity*);  // _no swig_
void UpdateIfSonOpen(Activity*);  // _no swig_
void UpdateIfSonClosed(Activity*);  // _no swig_
void UpdateIfSonNoLongerOpenable(Activity*);  // _no swig_

bool         IsOrMayBeUniqueLast(Activity*);
bool         HasOperationNotBegunAtMaxBegDate();
bool         HasOperationNotBegun();

          /// 
void Iterate ();

protected :

};    // _end class_

//@}  // fin iteration

//----------------------------------------------------------------------------
// ACTIVITY_OPTIONAL 
//----------------------------------------------------------------------------
/**@name Activités optionnelles
\label{classeActivityOptional}\index{opérateur de composition!optionnalité}
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_Activity.h} \item opérateur de composition : {\em optional} \end{itemize}

Cette classe permet d'exprimer qu'une activité (éventuellement non primitive) dite "optionnelle" peut être réalisée ou non, sans que cela constitue un défaut de contrôle du système. La non réalisation correspond à une des trois situations suivantes :\\
\begin{itemize} 
\item l'activité optionnelle n'a pas pu être ouverte dans l'intervalle entre la date de début au plus tôt et la date de début au plus tard ;
\item il faudrait ouvrir l'activité optionnelle par propagation, mais sa condition d'ouverture n'est pas satisfaite à cet instant : par exemple on doit pouvoir ouvrir une activité composante d'une meeting dès que la précédente devient closed.
\item l'activité optionnelle a pu être ouverte dans l'intervalle entre la date de début au plus tôt et la date de début au plus tard, mais en l'instant où la condition de fermeture est satisfaite, aucune opération en jeu dans l'activité n'a connu un début de réalisation (degré de progression = 0).  Cette situation peut provenir de l'indisponibilité constante des ressources requises, ou bien de la non-satisfaction constante de la condition de faisabilité des opérations en jeu.
\end{}itemize}

Une activité composante à une profondeur quelconque d'une activité optionnelle ne peut  figurer ailleurs dans le plan d'activités que comme composante à une profondeur quelconque d'une activité optionnelle. En d'autres termes une activité ne peut pas être déclarée ici optionnelle et là obligatoire.\\

{\bf descripteurs :} \\

{\scriptsize 
\noindent\begin{tabular}{lllll}
\end{tabular}
}\\

{\bf méthodes :} \ \\

{\scriptsize 
\noindent\begin{tabular}{llll}
\end{tabular} 
}

*/
//@{
          ///
//----------------------------------------------------------------------------
class ActivityOptional: public NonPrimitiveActivity  // _no swig_
{
public:

          /// Constructeur.
ActivityOptional();

//TableauInt* OpeningWindow();

int  GetMinBegDateFromDown();
int  GetMaxBegDateFromDown();
int  GetMinEndDateFromDown();
int  GetMaxEndDateFromDown();

int  PropagateMinBegDateToSon(Activity* pSon);
int  PropagateMaxBegDateToSon(Activity* pSon);
int  PropagateMinEndDateToSon(Activity* pSon);
int  PropagateMaxEndDateToSon(Activity* pSon);


bool CheckSonsIfWaiting();  // _no swig_
bool CheckSonsIfWaiting(Activity*);  // _no swig_
bool CheckSonsIfOpen();  // _no swig_
bool CheckSonsIfOpen(Activity*);  // _no swig_
bool CheckSonsIfClosed();  // _no swig_
bool CheckSonsIfClosed(Activity*);  // _no swig_

bool CheckIfSonWaiting(Activity*);  // _no swig_
bool CheckIfSonOpen(Activity*);  // _no swig_
bool CheckIfSonClosed(Activity*);  // _no swig_

void PropagateWaitingToSons(Activity*);  // _no swig_
void PropagateOpenToSons(Activity*);  // _no swig_
void PropagateClosedToSons(Activity*);  // _no swig_

void UpdateIfSonWaiting(Activity*);  // _no swig_
void UpdateIfSonOpen(Activity*);  // _no swig_
void UpdateIfSonClosed(Activity*);  // _no swig_
void UpdateIfSonNoLongerOpenable(Activity*);  // _no swig_

bool         IsOrMayBeUniqueLast(Activity*);
bool         HasOperationNotBegunAtMaxBegDate();
bool         HasOperationNotBegun();

protected :

};   // _end class_ 

//@}  // fin iteration

//@}  // fin opérateurs de composition
 
//@}  // fin Les activités (ouvert dans C_Activity.h)

#endif  // _C_ACTIVITY_NONPRIMITIVE_HH

 
