//============================================================================
// FILE: C_AggregatedResource.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des ressources agrégées
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_AGGREGATEDRESOURCE_HH
#define _C_AGGREGATEDRESOURCE_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "C_Resource.h"
#include "C_CoreTypes.h"
#include "C_ResourceCapacity.h"

// EXTERNAL REFERENCES

/**@name Les ressources agrégées 
\begin{itemize} \item Directive d'inclusion : 
[P]\#include C_AggregatedResource.h[/P]
\end{itemize}

Une ressource agrégée est une ressource qui est l'assemblage d'autres ressources agrégées et/ou atomiques ([C]SingleResource[/C]), qu'on appelle les "ressources éléments".\\

Il y a deux classes-mères pour les ressources agrégées :
\begin{itemize}
\item les resources agrégées homogènes ([C]HomogeneousAggregatedResource[/C]), qui sont des entités sans composants, mais avec des éléments, tous instances d'une même classe à des degrés de profondeur qui peuvent être variés ;
\item les resources agrégées hétérogènes([C]HeterogeneousAggregatedResource[/C]), qui sont des entités sans éléments, mais avec des composants qui sont des instances de différentes classes.
\end{itemize}

L'état de disponibilité et, le cas échéant, la capacité et le degré de disponibilité d'une ressource agrégée sont chacun fonction des mêmes propriétés sur les ressources éléments. Cette fonction est spécifique de chaque spécialisation de ressource agrégée.\\
 
Une ressource agrégée peut figurer ou non dans le pool de ressources qui est un composant ([C]ResourcePoolDisjunction[/C]) du système opérant ([C]OperatingSystem[/C]). Si ce n'est pas le cas, les ressources atomiques en position d'éléments ou de composants y figurent nécessairement. Il arrive notamment qu'une ressource agrégée (par exemple une équipe de travailleurs) soit constituée pour une période délimitée, et soit détruite en fin de période. Cette ressource n'est pas nécessairement placée dans le pool des ressources (percues comme permanentes) du système, mais ses travailleurs individuels doivent être et rester dans ce pool.
*/ 
//@{

/**@name Ressources agrégées homogènes\\

On rappelle qu'une resource agrégée homogène, est une entité sans composants, mais avec des éléments, tous instances d'une même classe à des degrés de profondeur qui peuvent être variés.\\

Soit par exemple les classes de ressources [C]MySR1[/C] et [C]MySR2[/C] héritant de la sous-classe [C]MySR[/C] de [C]SingleResource[/C]. Et soit [C]MyHAR[/C] une sous-classe de [C]HomogeneousAggregatedResource[/C]. On peut alors écrire de la manière suivante le constructeur de [C]MyHAR[/C] :\\
[P]
\begin{verbatim}
class MyHAR : public HomogeneousAggregatedResource { public: MyHAR(); };

MyHAR::MyHAR() {
  SetMetaClass(MY_H_A_R);
  //  SetParentclassId(HOMOGENEOUS_AGGREGATED_RESOURCE); // pour mémoire
  AddInstanceToClass();
  ElementClassId(MY_S_R);  // la contrainte sur la classe des éléments
}
\end{verbatim}
[/P]

Puisque les éléments doivent être des instances de [C]MySR[/C], on peut construire une instance de [C]MyHAR[/C] de la manière suivante, avec deux éléments qui ont bien la classe [C]MySR[/C] comme ancêtre commun :\\
[P]
\begin{verbatim}{
  //...
  MyHAR* pHAR = new MyHAR();

  MySR* pSR_a = new MySR();
  pHAR->AddElement(pSR_a);

  MySR1* pSR1_a = new MySR1();
  pHAR->AddElement(pSR1_a);
}
\end{verbatim}
[/P]

Dans cet exemple, la ressource agrégée n'a pas encore d'éléments de la classe [C]MySR2[/C].

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny C} & {\tiny int} & {\tiny [C]IsSingle[/C] hérité} & {\tiny 0} & {\tiny }
\end{tabular}
*/ 
//@{

/**@name La classe de base des ressources agrégées homogènes\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]HomogeneousAggregatedResource[/C]} & {\tiny HOMOGENEOUS_AGGREGATED_RESOURCE} & {\tiny homogeneousAggregatedResource} & {\tiny RESOURCE} & {\tiny }
\end{tabular}

 */
//@{
        ///
class HomogeneousAggregatedResource : public Resource  
{
public:
          // Constructeur de base.
  HomogeneousAggregatedResource();
          // Constructeur par copie.
  HomogeneousAggregatedResource(const HomogeneousAggregatedResource&);
          // Destructeur.
  ~HomogeneousAggregatedResource();

/**@name  [SC]Test des contraintes de disponibilité[/SC]*/
//@{
         /// Renvoie {\em faux} si le même test échoue sur une ressource-élément, et {\em vrai} sinon.
Boolean CheckTimeAvailabilityConstraint();  // _no swig_

         // non réalisée
//Boolean CheckCapacityAvailabilityConstraints();  // _no swig_ 
//@} 

 };
//@}  // fin classe de base AggrRsc

//===========================================================================

/**@name Agrégat homogène de ressources à état discret\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]HomogeneousAggregatedDiscreteStateResource[/C]} & {\tiny HOMOGENEOUS_AGGREGATED_DISCRETE_STATE_RESOURCE} & {\tiny homogeneousAggregatedDiscreteStateResource} & {\tiny } & {\tiny }
\end{tabular}
*/ 
//@{
        ///
class HomogeneousAggregatedDiscreteStateResource : public HomogeneousAggregatedResource
{
public:
          // Constructeur de base.
  HomogeneousAggregatedDiscreteStateResource();
          // Constructeur par copie.
  HomogeneousAggregatedDiscreteStateResource(const HomogeneousAggregatedDiscreteStateResource&);
          // Destructeur.
  ~HomogeneousAggregatedDiscreteStateResource();

/**@name  [SC]Test des contraintes de disponibilité[/SC]*/
//@{
         /// Renvoie {\em faux} si le même test échoue sur une ressource-élément, et {\em vrai} sinon.\index{méthodes!CheckStateAvailabilityConstraints}
Boolean CheckStateAvailabilityConstraints();  // _no swig_

//Boolean CheckElementCapacityAvailabilityConstraints();  // _no swig_ 

         /// Appel normalement inapproprié. Emet un avertissement et renvoie {\em vrai}.
Boolean CheckCapacityAvailabilityConstraints();  // _no swig_
//@}  

 };

//@}  // fin Agrégat homogène de ressources à état discret

//===========================================================================

/**@name Agrégat homogène de ressources à état discret réutilisables\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]HomogeneousAggregatedDiscreteStateReusableResource[/C]} & {\tiny HOMOGENEOUS_AGGREGATED_DISCRETE_STATE_REUSABLE_RESOURCE} & {\tiny homogeneousAggregatedDiscreteStateReusableResource} & {\tiny } & {\tiny }
\end{tabular}

Cette classe ne possède ni descripteur, ni méthode, ni service particulier.\\

Elle peut servir dans une application à distinguer de manière pratique, parmi les instances de [C]HomogeneousAggregatedDiscreteStateResource[/C], celles réutilisables de celles consomables.
*/ 
//@{
        ///
class HomogeneousAggregatedDiscreteStateReusableResource : public HomogeneousAggregatedDiscreteStateResource
{
public:
          // Constructeur de base.
  HomogeneousAggregatedDiscreteStateReusableResource();
          // Constructeur par copie.
  HomogeneousAggregatedDiscreteStateReusableResource(const HomogeneousAggregatedDiscreteStateReusableResource&);
          // Destructeur.
  ~HomogeneousAggregatedDiscreteStateReusableResource();
 };

//@}  // fin Agrégat homogène de ressources à état discret réutilisables

//===========================================================================

/**@name Agrégat homogène de ressources à état discret consommables\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]HomogeneousAggregatedDiscreteStateConsumableResource[/C]} & {\tiny HOMOGENEOUS_AGGREGATED_DISCRETE_STATE_CONSUMABLE_RESOURCE} & {\tiny homogeneousAggregatedDiscreteStateConsumableResource} & {\tiny } & {\tiny }
\end{tabular}

Cette classe ne possède ni descripteur, ni méthode, ni service particulier.\\

Elle peut servir dans une application à distinguer de manière pratique, parmi les instances de [C]HomogeneousAggregatedDiscreteStateResource[/C], celles consomables de celles réutilisables.
*/ 
//@{
        ///
class HomogeneousAggregatedDiscreteStateConsumableResource : public HomogeneousAggregatedDiscreteStateResource
{
public:
          // Constructeur de base.
  HomogeneousAggregatedDiscreteStateConsumableResource();
          // Constructeur par copie.
  HomogeneousAggregatedDiscreteStateConsumableResource(const HomogeneousAggregatedDiscreteStateConsumableResource&);
          // Destructeur.
  ~HomogeneousAggregatedDiscreteStateConsumableResource();
 };

//@}  // fin Agrégat homogène de ressources à état discret consommables

//===========================================================================

/**@name Agrégat homogène de ressources à capacité\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]HomogeneousAggregatedCapacitatedResource[/C]} & {\tiny HOMOGENEOUS_AGGREGATED_CAPACITATED_RESOURCE} & {\tiny homogeneousAggregatedCapacitatedResource} & {\tiny } & {\tiny }
\end{tabular}
*/ 
//@{
        ///
class HomogeneousAggregatedCapacitatedResource : public HomogeneousAggregatedResource
{
public:
          // Constructeur de base.
  HomogeneousAggregatedCapacitatedResource();
          // Constructeur par copie.
  HomogeneousAggregatedCapacitatedResource(const HomogeneousAggregatedCapacitatedResource&);
          // Destructeur.
  ~HomogeneousAggregatedCapacitatedResource();

/**@name  [SC]Test des contraintes de disponibilité[/SC]*/
//@{
          /// Appel normalement inapproprié. Emet un avertissement et renvoie {\em vrai}.
         Boolean CheckStateAvailabilityConstraints();  // _no swig_

//Boolean CheckElementCapacityAvailabilityConstraints();  // _no swig_ 

          /// Renvoie {\em vrai} si la ressource satisfait toutes les contraintes de disponibilité liées à sa capacité qui lui sont attachées.\index{méthodes!CheckCapacityAvailabilityConstraints} 
Boolean CheckCapacityAvailabilityConstraints();  // _no swig_ 
//@}

/**@name OPERATION */
//@{
          /// Etablit la capacité (multidimensionnelle) offerte par la ressource agrégée, à partir des capacités (multidimensionnelles) des ressources (agrégées ou atomiques) qui sont ses éléments.\index{méthodes!SetResourceCapacitySupplyFromElements}
void SetResourceCapacitySupplyFromElements();  // _no swig_ 

//@}
 }; 

//@}  // fin Agrégat homogène de ressources à capacité

//===========================================================================

/**@name Agrégat homogène de ressources à capacité réutilisables\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]HomogeneousAggregatedCapacitatedReusableResource[/C]} & {\tiny HOMOGENEOUS_AGGREGATED_CAPACITATED_REUSABLE_RESOURCE} & {\tiny homogeneousAggregatedCapacitatedReusableResource} & {\tiny } & {\tiny }
\end{tabular}

Cette classe ne possède ni descripteur, ni méthode, ni service particulier.\\

Elle peut servir dans une application à distinguer de manière pratique, parmi les instances de [C]HomogeneousAggregatedCapacitatedResource[/C], celles réutilisables de celles consomables.
*/ 
//@{
        ///
class HomogeneousAggregatedCapacitatedReusableResource : public HomogeneousAggregatedCapacitatedResource
{
public:
          // Constructeur de base.
  HomogeneousAggregatedCapacitatedReusableResource();
          // Constructeur par copie.
  HomogeneousAggregatedCapacitatedReusableResource(const HomogeneousAggregatedCapacitatedReusableResource&);
          // Destructeur.
  ~HomogeneousAggregatedCapacitatedReusableResource(); 

 }; 

//@}  // fin Agrégat homogène de ressources à capacité réutilisables

//===========================================================================

/**@name Agrégat homogène de ressources à capacité consommables\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]HomogeneousAggregatedCapacitatedConsumableResource[/C]} & {\tiny HOMOGENEOUS_AGGREGATED_CAPACITATED_CONSUMABLE_RESOURCE} & {\tiny homogeneousAggregatedCapacitatedConsumableResource} & {\tiny } & {\tiny }
\end{tabular}

Cette classe ne possède ni descripteur, ni méthode, ni service particulier.\\

Elle peut servir dans une application à distinguer de manière pratique, parmi les instances de [C]HomogeneousAggregatedCapacitatedResource[/C], celles consomables de celles réutilisables.
*/ 
//@{
        ///
class HomogeneousAggregatedCapacitatedConsumableResource : public HomogeneousAggregatedCapacitatedResource
{
public:
          // Constructeur de base.
  HomogeneousAggregatedCapacitatedConsumableResource();
          // Constructeur par copie.
  HomogeneousAggregatedCapacitatedConsumableResource(const HomogeneousAggregatedCapacitatedConsumableResource&);
          // Destructeur.
  ~HomogeneousAggregatedCapacitatedConsumableResource(); 

 };

//@}  // fin Agrégat homogène de de ressources à capacité consommables

//===========================================================================

/**@name Agrégat homogène de performers\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]HomogeneousAggregatedPerformer[/C]} & {\tiny HOMOGENEOUS_AGGREGATED_PERFORMER} & {\tiny homogeneousAggregatedPerformer} & {\tiny } & {\tiny }
\end{tabular}

Voir dans la description de la classe le mode de calcul de la puissance. Noter que {\em 0.} est la valeur par défaut.

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny V} & {\tiny float} & {\tiny [C]Power[/C] POWER power} & {\tiny 0.0} & {\tiny {\em la puissance du performer agrégé}
\end{tabular}
*/ 
//@{
        ///
class HomogeneousAggregatedPerformer : public HomogeneousAggregatedDiscreteStateReusableResource
{
public:
          // Constructeur de base.
  HomogeneousAggregatedPerformer();
          // Constructeur par copie.
  HomogeneousAggregatedPerformer(const HomogeneousAggregatedPerformer&);
          // Destructeur.
  ~HomogeneousAggregatedPerformer();

        /// Renvoie la puissance du performer agrégé. Pour une telle ressource agrégée homogène, c'est la somme des valeurs retournées par [Y]GetPower[/Y] pour chacun des éléments (atomiques ou non).
  float GetPower();  // + JPR 111027
 };

//@}  // fin Agrégat homogène de performers

//===========================================================================

/**@name Equipe homogène de travailleurs\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]HomogeneousLaborTeam[/C]} & {\tiny HOMOGENEOUS_LABOR_TEAM} & {\tiny homogeneousLaborTeam} & {\tiny } & {\tiny }
\end{tabular}

Une instance de cette classe est censée recevoir comme éléments soit des instances de [C]SingleWorker[/C], soit des équipes homogènes de travailleurs. Aucun avertissement n'est émis, ni aucune exception n'est lancée par le moteur si cette convention n'est pas respectée.
*/ 
//@{
        ///
class HomogeneousLaborTeam : public HomogeneousAggregatedPerformer
{
public:
          // Constructeur de base.
  HomogeneousLaborTeam();
          // Constructeur par copie.
  HomogeneousLaborTeam(const HomogeneousLaborTeam&);
          // Destructeur.
  ~HomogeneousLaborTeam();
 };

//@}  // fin Equipe homogène de travailleurs

//===========================================================================

//@}  // fin HomogeneousAggregatedResource

//===========================================================================

/**@name Ressources agrégées hétérogènes\\

On rappelle qu'une resource agrégée hétérogène, est une entité sans éléments, mais avec des composants, tous instances de classe différentes à des degrés de profondeur qui peuvent être variés.\\

Soit par exemple les classes de ressources [C]MySRA[/C] et [C]MySRB[/C] ayant [C]SingleResource[/C] comme première classe ancêtre commune. Et soit [C]MyHAR[/C] une sous-classe de [C]HeterogeneousAggregatedResource[/C]. On peut alors écrire de la manière suivante le constructeur de [C]MyHAR[/C] :\\
[P]
\begin{verbatim}
class MyHAR : public HeterogeneousAggregatedResource { public: MyHAR(); };

MyHAR::MyHAR() {
  SetMetaClass(MY_H_A_R);
  //  SetParentclassId(HETEROGENEOUS_AGGREGATED_RESOURCE); // pour mémoire
  AddInstanceToClass();
  AddComponentClassId(MY_S_R_A);  // la classe du premier composant
  AddComponentClassId(MY_S_R_B);  // la classe du second composant
}
\end{verbatim}
[/P]

On peut construire une instance de [C]MyHAR[/C] de la manière suivante :\\
[P]
\begin{verbatim}{
  //...
  MyHAR* pHAR = new MyHAR();

  MySRA* pSRA = new MySRA();
  pHAR->AddComponent(pSRA);

  MySRB* pSRB = new MySRB();
  pHAR->AddComponent(pSRB);
}
\end{verbatim}
[/P]
*/
//@{ 

/**@name La classe de base des ressources agrégées hétérogènes\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]HeterogeneousAggregatedResource[/C]} & {\tiny HETEROGENEOUS_AGGREGATED_RESOURCE} & {\tiny heterogeneousAggregatedResource} & {\tiny } & {\tiny }
\end{tabular}
*/
//@{
        ///
class HeterogeneousAggregatedResource : public Resource {
public:
          // Constructeur de base.
  HeterogeneousAggregatedResource();
          // Constructeur par copie.
  HeterogeneousAggregatedResource(const HeterogeneousAggregatedResource&);
          // Destructeur.
  ~HeterogeneousAggregatedResource();

/**@name  [SC]Test des contraintes de disponibilité[/SC] */
//@{
          /// Renvoie {\em faux} si le même test échoue sur une ressource-composant, et {\em vrai} sinon.\index{méthodes!CheckTimeAvailabilityConstraints} 
Boolean CheckTimeAvailabilityConstraint();  // _no swig_

          /// Renvoie {\em faux} si le même test échoue sur une ressource-composant, et {\em vrai} sinon ; si un composant est n'est ni une ressource atomique à état discret, ni une ressorce agrégée hétérogène, ni une ressource agrégée homogène à état discret, on l'ignore sans rendre {\em faux} le test global. \index{méthodes!CheckTimeAvailabilityConstraints}
Boolean CheckStateAvailabilityConstraints();  // _no swig_

          /// Renvoie {\em faux} si le même test échoue sur une ressource-composant, et {\em vrai} sinon.\index{méthodes!CheckCapacityAvailabilityConstraints} 
Boolean CheckCapacityAvailabilityConstraints();  // _no swig_ 
//@}
 };

//@}  // fin classe de base HeterogeneousAggregatedResource

/**@name Agrégat hétérogène de performers\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]HeterogeneousAggregatedPerformer[/C]} & {\tiny HETEROGENEOUS_AGGREGATED_PERFORMER} & {\tiny heterogeneousAggregatedPerformer} & {\tiny } & {\tiny }
\end{tabular}

Voir dans la description de la classe le mode de calcul de la puissance. Noter que {\em 1.} est la valeur par défaut.

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
 {\tiny V} & {\tiny float} & {\tiny [C]Power[/C] POWER power} & {\tiny {\em1.0}} & {\tiny la puissance du performer agrégé}
\end{tabular}
*/ 
//@{
        ///
class HeterogeneousAggregatedPerformer : public HeterogeneousAggregatedResource
{
public:
          // Constructeur de base.
  HeterogeneousAggregatedPerformer();
          // Constructeur par copie.
  HeterogeneousAggregatedPerformer(const HeterogeneousAggregatedPerformer&);
          // Destructeur.
  ~HeterogeneousAggregatedPerformer();

        /// Renvoie la puissance du performer. Pour une telle ressource agrégée hétérogène, c'est la valeur du descripteur [C]Power[/C]. La puissance n'est donc pas calculée à partir de celle des composants. Le développeur ou l'utilisateur doit donc explicitement changer la valeur par défaut. Ce service est exploité par le moteur dans le calcul de la puissance d'un performer aggrégé homogène.
  float GetPower();  // + JPR 111027
 };

//@}  // fin Agrégat hétérogène de performers

/**@name Equipe hétérogène de travailleurs\\

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]HeterogeneousAggregatedLaborTeam[/C]} & {\tiny HETEROGENEOUS_AGGREGATED_LABOR_TEAM} & {\tiny heterogeneousAggregatedLaborTeam} & {\tiny } & {\tiny }
\end{tabular}

Une instance de cette classe est censée recevoir comme composants soit des instances de [C]SingleWorker[/C], soit des équipes homogènes ou hétérogènes de travailleurs. Aucun avertissement n'est émis, ni aucune exception n'est lancée par le moteur si cette convention n'est pas respectée.
*/ 
//@{
        ///
class HeterogeneousLaborTeam : public HeterogeneousAggregatedPerformer
{
public:
          // Constructeur de base.
  HeterogeneousLaborTeam();
          // Constructeur par copie.
  HeterogeneousLaborTeam(const HeterogeneousLaborTeam&);
          // Destructeur.
  ~HeterogeneousLaborTeam();
 };

//@}  // fin Equipe hétérogène de travailleurs

//@}  // fin Les ressources agrégées hétérogènes

//@}  // fin Les ressources agrégées 

// @}  // fin SingleResource +  ressources agrégées



#endif  // _C_AGGREGATEREDRESOURCE_HH
