//============================================================================
// FILE: O_InterfaceDataItem.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des classes des 'InterfaceDataItem' de Open-Diese
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : 
//=X===============X=============================X============================

#ifndef _O_INTERFACEDATAITEM_HH
#define _O_INTERFACEDATAITEM_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "O_CoreTypes.h"

// /X///////////////X/////////////////////////////X////////////////////////////
//
//  INTERFACE_DATA_ITEM
//
// /X///////////////X/////////////////////////////X////////////////////////////

/**@name Structure de données à l'interface client / serveur
\label{classeInterfaceDataItem}
\begin{itemize} \item Directive d'inclusion : {\tt \#include O\_InterfaceDataItem.h} \end{itemize}

La bibliothèque \OD permet de développer un simulateur de système destiné à être placé dans une position de "serveur" vis-à-vis d'une procédure "maître". Celle-ci va contrôler le simulateur (c'est-à-dire décider le moment de son invocation et les bornes de la période simulée du système). La procédure maître va aussi transmettre des informations au simulateur, et en recevoir de lui. La présente classe est le véhicule de ces informations.\\

Chaque instance de cette classe (ou d'une classe déscendante définie et spécialisée au niveau de l'application) véhicule la valeur d'un descripteur d'une entité. Et c'est un ensemble de telles instances qui est communiqué, dans les deux sens, entre le programme maître et le simulateur.\\

Les instants auxquels la communication est opérée sont, exclusivement :
\begin{itemize}
\item le moment de l'appel, par le programme maître, de la méthode {\bf RunOpenSimulation} de la simulation courante ; à cet instant, le programme maître est mis en pause et "passe la main" au simulateur ;
\item le moment du "retour" de la méthode, c'est-à-dire celui où le simulateur "redonne la main" au programme maître.
\end{itemize}

La documentation de la classe de base précise les informations qui sont transmises dans chacun des deux cas.
 */
//@{

/**@name La classe de base 
Une instance de la classe {\bf InterfaceDataItem} est définie par 4 éléments :
\begin{itemize}
\item le nom d'instance de l'entité (instance de la classe {\bf Entity}) dont on veut véhiculer la valeur d'un descripteur
\item le descripteur évoqué dans l'item ci-dessus, spécifié par son nom de classe ou son symbole de classe
\item le type de la valeur véhiculée
\item une date, en format ISO 8601, option "aaaa-mm-qqThh:mi:sszzzzzz" (par exemple "2005-02-15T00:00:00+00:00")
\end{itemize}
Selon le sens du passage d'information, la date n'a pas la même sémantique :
\begin{itemize}
\item quand le programme maître transmet une valeur au simulateur, la date est celle à laquelle le simulateur doit réaliser l'affectation de valeur au descripteur de l'entité ;
\item quand le programme maître interroge le simulateur sur la valeur d'un descripteur, la date est celle à laquelle le simulateur doit capter la valeur du descripteur et la mémoriser jusqu'à la fin de l'épisode de simulation (voir page \pageref{episodeSimulation}). 
\end{itemize}
Respectivement pour ces deux sens de passage d'information :
\begin{itemize}
\item les méthodes {\bf SetInterfaceValue(..), argumentées par un entier, un nombre flottant, etc. selon le type, sont utilisées par le programme maître pour indiquer la valeur que le simulateur devra affecter à la date spécifiée ;
\item les méthodes {\bf Get<type>InterfaceValue() sont utilisées par le programme maître pour connaître la valeur (entière, flottante, etc. selon le type) que le simulateur a captée à la date spécifiée.
\end{itemize}

*/
//@{
        ///
class InterfaceDataItem : public Entity
{
public:

/**@name LIFECYCLE */
//@{ 
        /// Constructeur de base.
  InterfaceDataItem();
   
        /// Constructeur de base, avec indication du type de valeur (INT, ...) pour le descripteur concerné.
  InterfaceDataItem(int type);
   
        /// Constructeur, avec indication du type de valeur (INT, ...) pour le descripteur concerné, le nom d'instance de l'entité concernée et le nom de classe du descripteur. 
  InterfaceDataItem(int type, char* entityName, char* descName);
   
        /// Constructeur, avec indication du type de valeur (INT, ...) pour le descripteur concerné, le nom d'instance de l'entité concernée et le symbole de classe du descripteur. 
  InterfaceDataItem(int type, char* entityName, UserDescriptorId descId);
//@{ 
        /// Destructeur de base.
  ~InterfaceDataItem();
//@}

/**@name Etablissement du type de valeur pour le descripteur concerné */
//@{ 
        /// Affecte un type (INT, ...) à la structure, c'est-à-dire à la valeur du descripteur concerné. Ce type devient la valeur du descripteur {\bf TypeAttribute} de la structure.\index{méthodes!SetType - InterfaceDataItem}
  void SetType(int); 

        /// Renvoie le type du descripteur concerné par la structure, c'est-à-dire la valeur de son descripteur {\bf TypeAttribute}.\index{méthodes!GetType - InterfaceDataItem}
  int GetType();
//@}

/**@name Etablissement de la valeur du descripteur concerné 
Les méthodes {\bf SetInterfaceValue} affectent une valeur à la structure. Les types de valeurs possibles sont INT (entier), FLOAT (flottant simple précision), DOUBLE (flottant double précision), P_ENTITY (pointeur sur une entité), STRING (chaîner de caractères).\\
Elles vérifient que la valeur du descripteur {\bf TypeAttribute} de la structure correspond au type de la valeur placée en argument.\\
La valeur affectée ici à la structure est la valeur que le simulateur devra affecter au descripteur de l'entité.\index{méthodes!SetInterfaceValue - InterfaceDataItem}
*/
//@{ 

        /// 
  void SetInterfaceValue(int);

        /// 
  void SetInterfaceValue(float);

        /// 
  void SetInterfaceValue(double);

        /// 
  void SetInterfaceValue(Entity*);

        /// 
  void SetInterfaceValue(char*); 

        /// 
  void SetInterfaceValue(pIntTab*); 

        /// 
  void AddInterfaceValueElement(int);

        /// 
  void SetInterfaceValue(pFloatTab*); 

        /// 
  void AddInterfaceValueElement(float);
//@}

/**@name Accès à la valeur du descripteur concerné 
Les méthodes {\bf Get<type>InterfaceValue} renvoient la valeur attachée à la structure. Les seuls types de valeurs possibles sont les mêmes que pour les méthodes {\bf SetInterfaceValue}.\\
Elles lèvent une exception si le type de la valeur ne correspond pas à la valeur du descripteur {\bf TypeAttribute} de la structure.\\
La valeur accédée ici dans la structure est la valeur du descripteur de l'entité que le simulateur a captée et mémorisée.\index{méthodes!GetIntValue - InterfaceDataItem} 
*/
//@{ 
        /// 
   int GetIntInterfaceValue();

        /// 
   float GetFloatInterfaceValue();

        /// 
   double GetDoubleInterfaceValue();

        /// 
   Entity* GetEntityInterfaceValue();

        /// 
   char* GetStringInterfaceValue();

        /// 
   pIntTab* GetIntTabInterfaceValue();

        /// 
   pFloatTab* GetFloatTabInterfaceValue();
//@}

   //
   void IsFired(Boolean);
   //
   Boolean IsFired();

    // comptage Const/Dest
    static int   getNbConst() {return _nbConst;}
    static int   getNbDest() {return _nbDest;}    


/* *@name OPERATIONS */
// @{
// @}

 protected :
   Boolean isFired;    

   int static   _nbConst;
   int static   _nbDest;
};

//@}  // fin La classe de base

//@}  // fin Structure de données à l'interface client / serveur

//=X===============X=============================X============================



#endif  // _O_INTERFACEDATAITEM_HH
