//============================================================================
// FILE: DescValueSpec.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe des spécifications
//              de domaines de valeurs
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _DESCVALUESPEC_HH
#define _DESCVALUESPEC_HH

// SYSTEM INCLUDES
//
//#include <>                                  

// PROJECT INCLUDES
//
#include "CoreTypes.h"


/**@name CDPFORTEXSpécifications de domaines de valeurs
Cette classe permet de spécifier un domaine de valeurs, c'est-à-dire de définir ce domaine sans en expliciter toutes les valeurs {\em a priori}. Cette classe fournit des services pour deux objectifs :
\begin{itemize}
\item donner un domaine de valeurs admissibles à un descripteur constant ou variable, protégeant l'utilisateur d'une affectation de valeur ne satisfaisant pas la sémantique du descripteur (voir les services [Y]IsInDomain[/Y] et [Y]IsOutOfDomain[/Y] des descripteurs),
\item établir un ensemble d'entités répondant à un critère, dynamiquement en cours de simulation, c'est-à-dire sur la base des valeurs courantes des descripteurs. La présente classe est alors à exploiter conjointement avec la classe [C]EntitySpec[/C] (voir page 'Spécifications d'ensembles d'entités'). La spécification de domaine de valeur doit alors inclure (dans les arguments du constructeur ou par le service [Y]SetDescId[/Y]) le symbole d'une classe de descripteurs dont les entités visées sont dotées. 
\end{itemize}

La spécification d'un domaine de valeur consiste en :
\begin{itemize}
\item un symbole de classe de descripteur, identifiant une classe de descripteurs (constants ou variables), caractérisée (implicitement ou explicitement) par un ensemble de valeurs possibles,
\item un domaine de référence, qui sert à partager l'ensemble de valeurs possibles en deux sous-ensembles,
\item un opérateur, parmi IN et NOT\_IN, qui sert à décider lequel des deux sous-ensembles définit la spécification.
\end{itemize}

Cette classe n'est pas destinée à recevoir des instances directes, au contraire de ses sous-classes correspondant à chaque type de descripteur d'entités.\\

Pour les descripteurs à valeurs entières ou flottantes (simple ou double précision) le domaine de référence est un ensemble d'intervalles. Pour les descripteurs dont les valeurs sont des chaînes de caractères, le domaine de référence est une liste de chaînes de caractères. Pour les descripteurs dont les valeurs sont des pointeurs sur des entités, le domaine de référence est soit une liste de pointeurs sur des entités soit une liste de classe d'entités. \\

La méthode [Y]IsInDomain[/Y] indique si la valeur courante d'un descripteur d'une instance de [C]DescriptedEntity[/C] est ou non dans le domaine que cette spécification définit. Elle prend l'entité en argument. Cette méthode est invoquée par l'{\em expansion} d'une spécification d'ensemble d'entités, lorsque celle-ci est dotée de spécifications de domaines de valeurs (voir page 'Spécifications d'ensembles d'entités'). Par exemple, pour savoir si la valeur du descripteur à valeur entière et de symbole de classe {\small {\tt MY_DESC}} est ou non comprise, pour l'entité pointée par {\small {\tt pObj}}, entre 1 et 5, on peut écrire :
\begin{verbatim}
[P]  DescIntValueSpec pDVS = new DescIntValueSpec(MY_DESC, NOT_IN, 1, 5);
  pObj->SetValue(MY_DESC, 6);
  pDVS->IsInDomain(pObj);     // renvoie vrai
  pObj->SetValue(MY_DESC, 3);
  pDVS->IsOutOfDomain(pObj);  // renvoie vrai[/P]
\end{verbatim}

La méthode [Y]GetValueInDomain[/Y] renvoie une valeur tirée au hasard dans le domaine spécifié. Elle est exploitée par l'{\em expansion} d'une spécification d'ensemble d'entités lorsque celle-ci génère des instances désirées (mode INSTANTIATE) : les spécifications de domaines de valeurs permettent alors d'affecter les valeurs aux descripteurs des instances créées (voir aussi page 'Spécifications d'ensembles d'entités').
\begin{verbatim}
[P]  int lb = 1;
  int up = 5;
  DescIntValueSpec pDVS = new DescIntValueSpec(MY_DESC, IN, lb, ub);
  int N = pDVS->GetValueInDomain();
  printf("\ntirage aléatoire dans le domaine [%d %d] : %d", lb, ub, N)[/P]
\end{verbatim}
\begin{verbatim}
>>
tirage aléatoire dans le domaine [1 5] : 5
\end{verbatim}
*/
//@{

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name La classe de base des spécifications de domaines de valeurs
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include DescValueSpec.h
[/P]
\end{itemize}\index{spécification de domaines}\label{docDescValueSpec}
*/
//@{
        ///
class DescValueSpec 
{
public:

  /* *@name LIFECYCLE */
// @{
          // Constructeur de base. Non invoqué directement, mais seulement par les constructeurs des sous-classes.
  DescValueSpec(); // _no swig_
  DescValueSpec(const DescValueSpec&); // _no swig_
  
          // Destructeur. 
  ~DescValueSpec(); // _no swig_

// @}                             

//  OPERATORS

         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const DescValueSpec&) const {return TRUE;};
  Boolean         operator > (const DescValueSpec&) const {return TRUE;};


  /**@name [SC]Identification[/SC] */
//@{
         /// Attribut un nom de classe.
  void               ClassName(char*); // _no swig_
         /// Renvoie le nom de classe.
  char*              ClassName();
         /// Etablit le symbole de classe de la classe de descripteurs sur laquelle porte la présente spécification.
  void               SetDescId(UserDescriptorId);

         // Renvoie le symbole de classe de la classe de descripteurs sur laquelle porte la présente spécification.
  UserDescriptorId   GetDescId();

         /// Etablit l'opérateur relationnel utilisé par la présente spécification. 
   void              SetRelationalOperator(RelationalOperator);

         // Renvoie l'opérateur relationnel utilisé par la présente spécification. 
  RelationalOperator GetRelationalOperator();

         // Etablit le type de descripteurs sur laquelle porte la présente spécification. 
  void               SetValueType(ValueTypeId);

         // Renvoie le type de descripteurs sur laquelle porte la présente spécification. 
  ValueTypeId        GetValueType();
//@}                         

  /**@name OPERATIONS
\\

Ces deux méthodes sont définies pour chaque sous-classe de [C]DescValueSpec[/C].\\

La première retourne {\em vrai} si, pour l'entité passée en argument, la valeur courante du descripteur de symbole de classe retourné par [Y]GetDescId[/Y] correspond à la spécification définie par l'opérateur retourné par [Y]GetRelationalOperator[/Y] et par le domaine de référence, et {\em faux} sinon.\\

La seconde retourne {\em vrai} si, pour l'entité passée en argument, la valeur courante du descripteur de symbole de classe retourné par [Y]GetDescId[/Y] ne correspond pas à la spécification définie par l'opérateur et le domaine de référence, et {\em faux} sinon.
\index{méthodes!IsInDomain}
*/
//@{ 
         /// 
  Boolean          IsInDomain(DescriptedEntity*);  // virtual : sup JPR 050407
         /// 
  Boolean          IsOutOfDomain(DescriptedEntity*);  // virtual : sup JPR 050407
//@}   

    // comptage Const/Dest
    static int   getNbConst() {return _nbConst;}
    static int   getNbDest() {return _nbDest;}     

    // comptage instances
    // DIESE.4.4 : on ne gère plus la liste des instances

protected:

   int static   _nbConst;
   int static   _nbDest;   
  
  char*                       mClassName;
  UserDescriptorId            mDescId;
  RelationalOperator          mRelOp;
  ValueTypeId                 mValueType;

private:
}; // _end class_

//@} // fin classe de base

/**@name Sous-classes de spécification de domaine selon le type de la valeur */
//@{  // ss_classes typées

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Spécification de domaine de valeurs entières */
//@{
        /// 
class DescIntValueSpec : public DescValueSpec
{
public:

/**@name LIFECYCLE */
//@{

          /// Constructeur de base.
  DescIntValueSpec(); // _no swig_

          /// Constructeur. L'argument doit être IN ou NOT\_IN.
  DescIntValueSpec(RelationalOperator); // _no swig_

          /// Constructeur. Le second argument doit être IN ou NOT\_IN.
  DescIntValueSpec(UserDescriptorId, RelationalOperator); // _no swig_

          // Constructeur. Le second argument doit être IN ou NOT\_IN. Le dernier argument devient les bornes inférieure et supérieure de l'intervalle "inférieur" du domaine.
  DescIntValueSpec(UserDescriptorId, RelationalOperator, int); // _no swig_

          /// Constructeur. Le second argument doit être IN ou NOT\_IN. Les deux derniers arguments sont les bornes de l'intervalle "inférieur" du domaine.
  DescIntValueSpec(UserDescriptorId, RelationalOperator, int, int); // _no swig_

          /// Constructeur par copie. Les intervalles sont copiés : le domaine de la copie ne pointe pas sur les mêmes intervalles que celui de la spécification originale.
   DescIntValueSpec(const DescIntValueSpec&); // _no swig_

          /// Destructeur. Les intervalles sont détruits.
   ~DescIntValueSpec();
//@}

  /**@name [SC]Manipulation du domaine[/SC] */
//@{

         /// Ajoute une valeur entière au domaine (en fait un intervalle dont les deux bornes sont égales), après celles déjà présentes.
void               AddSubDomain(int); // _no swig_
         /// Ajoute un intervalle au domaine, après ceux déjà présents.
void               AddSubDomain(int, int);

         /// Insère une valeur entière au domaine, en respectant l'ordre croissant des valeurs.
void               InsertSubDomain(int); // _no swig_
         /// Insère un intervalle au domaine, en respectant l'ordre croissant des bornes inférieures.
void               InsertSubDomain(int, int);

         /// Renvoie une valeur du domaine.
int                GetValueInDomain();

         //
pIntIntervalTab*    GetDomain();

         //
Boolean            IsInDomain(DescriptedEntity*);
         //
bool               IsOutOfDomain(int);

         /// Trie les intervalles dans l'ordre croissant de leurs bornes inférieures.\index{méthodes!SortOnLowBound - DescIntValueSpec} 
  void             SortOnLowBound();

         /// Remplace la série d'intervalles, triée par ordre croissant des bornes inférieures, par une autre telle que tous les éléments sont disjoints.\index{méthodes!ReduceToDisjunction - DescIntValueSpec} 
  void             ReduceToDisjunction();
//@}

protected:
  
  pIntIntervalTab*             mDomain;

private:
};   // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Spécification de domaine de valeurs flottantes */
//@{
        /// 
class DescFloatValueSpec : public DescValueSpec
{
public:

/**@name LIFECYCLE */
//@{

          /// Constructeur de base.
  DescFloatValueSpec(); // _no swig_

          /// Constructeur. L'argument doit être IN ou NOT\_IN.
  DescFloatValueSpec(RelationalOperator); // _no swig_
          /// Constructeur. Le second argument doit être IN ou NOT\_IN.
  DescFloatValueSpec(UserDescriptorId, RelationalOperator); // _no swig_
          // Constructeur. Le second argument doit être IN ou NOT\_IN. Le dernier argument devient les bornes inférieure et supérieure de l'intervalle "inférieur" du domaine.
  DescFloatValueSpec(UserDescriptorId, RelationalOperator, float); // _no swig_
          /// Constructeur. Le second argument doit être IN ou NOT\_IN. Les deux derniers arguments sont les bornes de l'intervalle "inférieur" du domaine.
  DescFloatValueSpec(UserDescriptorId, RelationalOperator, float, float); // _no swig_

          /// Constructeur par copie. Les intervalles sont copiés : le domaine de la copie ne pointe pas sur les mêmes intervalles que celui de la spécification originale.
   DescFloatValueSpec(const DescFloatValueSpec&); // _no swig_

          /// Destructeur. Les intervalles sont détruits.
   ~DescFloatValueSpec(); // _no swig_
//@}

  /**@name [SC]Manipulation du domaine[/SC] */
//@{

         /// Ajoute un intervalle au domaine, après ceux déjà présents..
void               AddSubDomain(float, float);
         /// Insère un intervalle au domaine, en respectant l'ordre croissant des bornes inférieures.
void               InsertSubDomain(float, float);

         /// Renvoie une valeur du domaine.
float              GetValueInDomain();

         //
pFloatIntervalTab*    GetDomain();

         //
Boolean            IsInDomain(DescriptedEntity*);
         //
bool               IsOutOfDomain(float);

         /// Trie les intervalles dans l'ordre croissant de leurs bornes inférieures.\index{méthodes!SortOnLowBound - DescFloatValueSpec} 
  void             SortOnLowBound();

         /// Remplace la série d'intervalles, triée par ordre croissant des bornes inférieures, par une autre telle que tous les éléments sont disjoints.\index{méthodes!ReduceToDisjunction - DescFloatValueSpec} 
  void             ReduceToDisjunction();
//@}

protected:
  
  pFloatIntervalTab*             mDomain;

private:
}; // _end class_

//@} // fin classe  

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Spécification de domaine de valeurs flottantes en double précision */
//@{
        /// 
class DescDoubleValueSpec : public DescValueSpec
{
public:

/**@name LIFECYCLE */
//@{

          /// Constructeur de base.
  DescDoubleValueSpec(); // _no swig_

          /// Constructeur. L'argument doit être IN ou NOT\_IN.
  DescDoubleValueSpec(RelationalOperator); // _no swig_

          /// Constructeur. Le second argument doit être IN ou NOT\_IN.
  DescDoubleValueSpec(UserDescriptorId, RelationalOperator); // _no swig_
          // Constructeur. Le second argument doit être IN ou NOT\_IN. Le dernier argument devient les bornes inférieure et supérieure de l'intervalle "inférieur" du domaine.
  DescDoubleValueSpec(UserDescriptorId, RelationalOperator, double); // _no swig_
          /// Constructeur. Le second argument doit être IN ou NOT\_IN. Les deux derniers arguments sont les bornes de l'intervalle "inférieur" du domaine.
  DescDoubleValueSpec(UserDescriptorId, RelationalOperator, double, double); // _no swig_

          /// Constructeur par copie. Les intervalles sont copiés : le domaine de la copie ne pointe pas sur les mêmes intervalles que celui de la spécification originale.
   DescDoubleValueSpec(const DescDoubleValueSpec&); // _no swig_

          /// Destructeur. Les intervalles sont détruits.
   ~DescDoubleValueSpec(); // _no swig_
//@}

  /**@name [SC]Manipulation du domaine[/SC] */
//@{

         /// Ajoute un intervalle au domaine, après ceux déjà présents..
void               AddSubDomain(double, double);
         /// Insère un intervalle au domaine, en respectant l'ordre croissant des bornes inférieures.
void               InsertSubDomain(double, double);

         /// Renvoie une valeur du domaine.
double             GetValueInDomain();

         //
pDoubleIntervalTab*    GetDomain();

         //
Boolean            IsInDomain(DescriptedEntity*);
         //
bool               IsOutOfDomain(double);

         /// Trie les intervalles dans l'ordre croissant de leurs bornes inférieures.\index{méthodes!SortOnLowBound - DescDoubleValueSpec}
  void             SortOnLowBound();

         /// Remplace la série d'intervalles, triée par ordre croissant des bornes inférieures, par une autre telle que tous les éléments sont disjoints.\index{méthodes!ReduceToDisjunction - DescDoubleValueSpec} 
  void             ReduceToDisjunction();
//@}


protected:
  
  pDoubleIntervalTab*             mDomain;

private:
}; // _end class_

//@} // fin classe  

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Spécification de domaine de valeurs de type 'chaine de caractères' */
//@{
        /// 
class DescStringValueSpec : public DescValueSpec
{
public:

/**@name LIFECYCLE */
//@{

          /// Constructeur de base.
  DescStringValueSpec(); // _no swig_

          /// Constructeur. L'argument doit être IN ou NOT\_IN.
  DescStringValueSpec(RelationalOperator); // _no swig_

          /// Constructeur. Le second argument doit être IN.
  DescStringValueSpec(UserDescriptorId, RelationalOperator); // _no swig_

          /// Constructeur. Le second argument doit être IN.
  DescStringValueSpec(UserDescriptorId, RelationalOperator, char*); // _no swig_

          /// Constructeur par copie. Les chaînes de caractères ne sont pas copiées : le domaine de la copie pointe sur les mêmes chaînes que celui de la spécification originale.
   DescStringValueSpec(const DescStringValueSpec&); // _no swig_

          /// Destructeur. Les chaînes sont détruites.
   ~DescStringValueSpec(); // _no swig_
//@}

  /**@name [SC]Manipulation du domaine[/SC] */
//@{

         /// Ajoute une chaîne de caractères au domaine.
void               AddSubDomain(char*);

         /// Renvoie une valeur du domaine.
char*              GetValueInDomain();

         //
TableauString*    GetDomain();

         //
Boolean            IsInDomain(DescriptedEntity*);
         //
bool               IsOutOfDomain(char*);
//@}

protected:
  
  TableauString                   mDomain;

private:
}; // _end class_

//@} // fin classe  

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Spécification de domaine de valeurs de type 'pointeur sur une entité' */
//@{
        ///
class DescEntityValueSpec : public DescValueSpec
{
public:

/**@name LIFECYCLE */
//@{

          /// Constructeur de base.
  DescEntityValueSpec(); // _no swig_

          /// Constructeur. L'argument doit être IN ou NOT\_IN. Le domaine devra être une liste d'instances d'entités.
  DescEntityValueSpec(RelationalOperator); // _no swig_

          /// Constructeur. Le second argument doit être IN. Le domaine devra être une liste d'instances d'entités.
  DescEntityValueSpec(UserDescriptorId, RelationalOperator); // _no swig_

          /// Constructeur. Le second argument doit être IN. Le domaine devra être une liste d'instances d'entités, parmi lesquelles celle passée en troisième argument.
  DescEntityValueSpec(UserDescriptorId, RelationalOperator, BasicEntity*); // _no swig_

          /// Constructeur. Le second argument doit être IN. Le domaine devra être une liste de symboles de classes d'entités, parmi lesquels celui passé en troisième argument. On peut donner une valeur négative au troisième argument, auquel cas le domaine devra être une liste de symboles de classes d'entités, ultérieurement ajoutés par [Y]AddSubDomain[/Y].
  DescEntityValueSpec(UserDescriptorId, RelationalOperator, UserClassId); // _no swig_

          /// Constructeur par copie. Les entités ne sont pas copiées : le domaine de la copie pointe sur les mêmes entités que celui de la spécification originale.
  DescEntityValueSpec(const DescEntityValueSpec&); // _no swig_

          /// Destructeur. Les entités ne sont pas détruites.
  ~DescEntityValueSpec(); // _no swig_
//@}

  /**@name [SC]Manipulation du domaine[/SC] */
//@{

         /// Ajoute une entité au domaine, définie comme un pointeur.
void               AddSubDomain(BasicEntity*);

         /// Ajoute une entité au domaine, définie par son nom d'instance.
void               AddSubDomain(char*); // _no swig_

         /// Désigne une classe d'entités comme domaine de valeurs.
void               AddSubDomain(UserClassId); // _no swig_

         /// Renvoie une valeur du domaine.
BasicEntity*            GetValueInDomain();

         //
pEntityTab*    GetDomain();

         /// Renvoie {\em vrai} si le domaine est défini comme un ensemble de classe d'entités, et {\em faux} s'il est défini comme un ensemble d'instances d'entités.\index{méthodes!IsDomainAClass}
Boolean            IsDomainAClass();

         //
Boolean            IsInDomain(DescriptedEntity*);
         //
bool               IsOutOfDomain(BasicEntity*);
//@}


protected:
  
  Boolean                 mIsDomainAClass;

  pEntityTab              mDomain;
  UserClassIdTab          mDomainClass;

private:
}; // _end class_ 

//@} // fin classe 

//-X///////////////X/////////////////////////////X////////////////////////////

//@}  // fin ss_classes typées

//@}  // fin DVS

#endif  // _DESCVALUESPEC_HH
