//============================================================================
// FILE: OutputSpec.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe des spécifications de sortie
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//          Chardon (INRA/PL et IE, Rennes) pour la sortie sur BD
//=X===============X=============================X============================

#ifndef _OUTPUTSPEC_HH
#define _OUTPUTSPEC_HH

// SYSTEM INCLUDES
//
#include <string>
#include <vector>
#ifdef _POSTGRESQL_
#include <libpq-fe.h>
#endif

// PROJECT INCLUDES
//
#include "CoreSimTime.h" 
#include "File.h"  // pour OutputMode


//-X///////////////X/////////////////////////////X////////////////////////////


/**@name CDPFORTEXLes spécifications de sortie
\label{defOutputSpec}
L'utilisateur d'un simulateur souhaite en général visualiser en direct ou sauvegarder la valeur de certaines variables en cours de simulation (par exemple l'historique des valeurs d'une variable ou bien certaines données synthétiques caractérisant l'ensemble de la simulation). A cet effet, il est possible de placer dans le code du simulateur des instructions dédiées à ces sorties. Un changement dans les souhaits de sorties implique alors une modification du code du simulateur. \\

Un simulateur construit avec \Db supporte de la part de l'utilisateur des spécifications de sorties changeantes d'une simulation à l'autre, et cela sans modification du code. Les demandes sont exprimées par l'utilisateur dans un fichier externe unique\index{spécifications de sortie (fichier)}. Les spécifications de sorties sont alors une "entrée" du simulateur. Pour exprimer les spécifications de sortie, un lexique et une syntaxe sont fournies à l'utilisateur. 
\noLF
Le programme principal du simulateur doit alors comprendre en son début (directement ou indirectement) un appel à la fonction [Y]ParseOutputSpecFile[/Y], laquelle interprète les lignes du fichier pour générer diverses structures internes du simulateur, qui sont des sous-classes de la classe [C]OutputSpec[/C].
\begin{verbatim} 
[P]main(int argc, char* argv[]) {
  ...
  pCurrentSim->SetOutDir("/home/durand/out/");
  ParseOutputspecFile(argv[3]);
  ...
  DisplayAllOutputSpecs();
  DeleteOutputSpecs();
}[/P]
\end{verbatim}
avec une ligne de commandes telle que :
\begin{verbatim} 
[P]./main  sim1.par  sim1.sim  sim1.osp  sim1.str  sim1.dir[/P]
\end{verbatim}
où [F]/home/durand/out/sim1.osp[/F] est le nom complet du fichier des directives de la simulation.\\

Le fichier pourra comprendre, une ou plusieurs fois, l'une ou l'autre des deux lignes suivantes pour, respectivement, activer et désactiver la trace des opérations effectuées :
\begin{verbatim}
[P]TRACE PARSING;
...
TRACE NO_PARSING;[/P]
\end{verbatim}

La présence en mémoire de ces structures [C]OutputSpec[/C] modifie légèrement l'action des fonctions normales de \Db (par exemple les fonctions d'affectation de valeurs aux descripteurs, ou bien la fonction de dépilement des événements, \ldots) : celles-ci enverront l'information à l'interface utilisateur pour être affichée à l'écran en temps réel, ou bien enregistreront l'information dans une structure appropriée qui sera exploitée après la fin de la simulation (une sauvegarde sur fichier, par exemple). Dans l'exemple ci-dessus, la fonction [Y]DisplayAllOutputSpecs[/Y] permet de traiter en une seule instruction toutes les sauvegardes, mais des services plus ciblés sont dans les pages suivantes.\\

{\large {\tt Notes importantes :}} Quand les informations sont sauvegardées dans des fichiers, ils sont situés, soit dans le répertoire dont le chemin est renvoyé par le message [Y]GetOutDir[/Y] adressé à la simulation courante (on aura préalablement adressé le message [Y]SetOutDir[/Y] pour affecter une valeur), soit à défaut dans le répertoire du module exécutable. Dans les deux cas, le nom à indiquer dans les spécifications de sortie est le nom simple du fichier.\\

Les mémoires allouées au titre des sauvegardes diverses (c'est-à-dire sur les classes d'entités, d'événements et de processus) peuvent être récupérées par un seul et même appel à la fonction [Y]DeleteOutputSpecs[/Y], typiquement placé vers la fin du programme principal, comme dans l'exemple ci-dessus.
*/
//@{

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name  La classe des spécifications de sortie
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include OutputSpec.h
[/P] 
\end{itemize}
Aucune instance n'est créée pour cette classe directement.\\

Selon que la spécification de sortie porte sur des entités, des événements ou des processus, c'est une instance d'une des sous-classes [C]EntityOutputSpec[/C], [C]EventOutputSpec[/C], [C]ProcessOutputSpec[/C] (respectivement) qui sera créée.\\

Toute spécification de sortie dans le fichier externe (et l'instance correspondante de [C]OutputSpec[/C] en mémoire) est caractérisée par le nom de la classe (d'entités, d'événements ou de processus) visée.\\

La méthode [Y]void TargetClassName(char*)[/Y] établit le nom de la classe dont les instances sont l'objet de la spécification de sortie. On rappelle que ce nom ne doit comporter que des caractères ASCII\index{ASCII}. Cette méthode n'est pas utilisée par le développeur de l'application : c'est l'interpréteur du fichier ([Y]ParseOutputSpecFile[/Y]) qui exploite ce service.\\

La méthode [Y]char* TargetClassName()[/Y] renvoie le nom de la classe dont les instances sont l'objet de la spécification de sortie. Cette méthode n'est pas généralement pas utilisée par le développeur de l'application : c'est le moteur de simulation, et les fonction d'affichage et de sauvegarde, qui exploitent ce service.\\ 
*/
//@{
  // / 
class OutputSpec 
{
public:

          // Constructeur de base. 
   OutputSpec(); 
          // Destructeur.                
   ~OutputSpec();

/* *@name ACCESS */
// @{
          // \index{méthodes!TargetClassName}
void              TargetClassName(char*);
          // Renvoie le nom de la classe dont les instances sont l'objet de la spécification de sortie.
char*             TargetClassName() const;
          // Renvoie le mode d'impression du temps.
TimeType             GetTimeType() const;
          // Etablit le mode d'impression du temps.
void                 SetTimeType(TimeType);
          // Renvoie le type de la spécification de sortie.
OutputSpecType    GetOutputSpecType() const;
// @}    


// INQUIRIES

    // comptage
    static int   getNbConst() {return _nbConst;}
    static int   getNbDest() {return _nbDest;}


  static pOutputSpecTab    mpInstanceTab; // Tableau des instances de OS
  static int	     mInstanceNumber;	// le nombre d'instance

protected:  
         
   int static      _nbConst;
   int static      _nbDest;
   TimeType        mTimeType;
   char*           mTargetClassName;
   OutputSpecType  mOutputSpecType;

};  // _end class_

//@} // fin classe 

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Sauvegarde d'informations sur les entités 
 L'utilisateur peut demander l'affichage à l'écran, ou la sauvegarde sur un ou plusieurs fichiers en écriture séquentielle, ou encore l'exportation dans une base de données, d'informations sur la vie des entités au cours de la simulation. Une telle demande porte :
\begin{itemize}
\item sur la structure des entités : date de création, ajout ou retrait de composants ou d'éléments
\item ou bien sur l'état des entités : valeur d'un descripteur constant ou changement de valeur des descripteurs variables. Le descripteur est à valeur entière, flottante, double, chaîne de caractères (moins de 50 caractères), entité ou tableau d'entités. Dans les deux derniers cas, la valeur sauvegardée est le nom d'instance de l'entité ou la liste des noms d'instances plaçée entre guillemets. 
\end{itemize}

Les trois liens ci-dessus orientent vers des exemples des diverses requêtes admises. Les mos-clés {\small {\tt CLOCK}} et {\small {\tt DATE_*}} y sont les symboles de l'énumération {\em TimeType} (voir page 'Les formats de date.').

L'ensemble des demandes portant sur une classe d'entités particulière fait l'objet d'une et une seule création d'instance de la classe [C]EntityOutputSpec[/C], caractérisée par la chaîne de caractères placée dans la spécification après les deux (ou trois si SIM_IDN est présent) premiers mots-clés, et accessible par la méthode [Y]TargetClassName[/Y]. Cette instance contient diverses structures de sauvegarde appropriées à chacun des types d'information désirés en sortie, et cela pour toutes les instances créées (et éventuellement supprimées) au cours de la simulation. Ces structures seront remplies au cours de la simulation, plus précisément au cours de la vie des instances de la classe d'entités concernée.\\

Au moment de la construction d'une entité (lors de l'attribution d'un nom de classe, par [Y]ClassName[/Y] exclusivement (voir page 'Fonctions globales/Gestion de la hiérarchie des classes d'entités'), \Db cherche s'il a été créé en début de simulation une instance de [C]EntityOutputSpec[/C] dont la chaîne de caractères renvoyée par [Y]TargetClassName[/Y] soit est identique au nom de classe de l'entité créée, soit correspond à une classe dont descend l'entité, directement ou indirectement. En cas de succès, la référence à l'instance de [C]EntityOutputSpec[/C] est attachée à l'entité, pour recevoir au cours de la simulation l'ensemble des informations demandées sur cette entité.\\

Les différents types d'informations demandées sur une classe d'entités peuvent être enregistrées sur plusieurs fichiers. Réciproquement, un même fichier peut recevoir des informations sur plusieurs classes, ou bien sur une classe d'entités et une classe de processus, etc.. Si un fichier de sortie est invoqué plusieurs fois dans un fichier de spécifications, le mode d'ouverture ({\small {\tt NEW}} ou {\small {\tt APPEND}}) doit être déterminé en connaissance qu'une spécification {\small {\tt NEW}} écrase les données déjà écrites sur le fichier de sortie.\\
Si un fichier de spécifications contient plusieurs demandes sur la même classe d'entités, c'est la dernière des directives {\small {\tt END}} ou {\small {\tt APPEND}} qui sera retenue pour l'ensemble des demandes.\\ 

La fonction [Y]DisplayEntityOutputSpecs[/Y] fournit une exploitation standard des informations enregistrées pour une classe d'entités dont le symbole de classe est passé en argument. Si l'argument est BD_ENTITY, ce sont les informations sur toutes classes d'entités qui sont traitées.  

*/
//@{
/**@name Sauvegarde sur fichier séquentiel

[P]
SAVE DESCRIPTOR  SIM_IDN "voiture" ALL "file_1.txt" NEW CLOCK "vitesse" 2 "position" 3;
[/P]

On demande la constitution d'un fichier ({\small {\tt SAVE}}) relatif à toutes les instances, directes ou indirectes, de la classe de le nom de classe "voiture". Tous les changements de valeur seront enregistrés ({\small {\tt ALL}}). Chaque ligne comprendra une date ({\small {\tt CLOCK}} : en unités de temps simulé depuis le début de la simulation), une valeur de vitesse (avec 2 décimales si la valeur est flottante) et une valeur de position (avec 3 décimales). Pour les valeurs entières, le moteur ramène le nombre pour les décimales à 0. On écrira dans le fichier [F]file_1.txt[/F] à partir de son début ({\small {\tt NEW}} : écrasement d'éventuelles données s'y trouvant déjà). Le fichier est créé s'il n'existe pas encore.\\
Les variables sont des descripteurs variables ou constants. La valeur d'un descripteur constant apparaîtra pour chaque date à laquelle au moins un descripteur variable a changé de valeur. \\
Le mot clé 'SIM_IDN' (le numéro de simulation) est optionnel. Voir commentaire à la page 'Sauvegarde sur base de données'.\\

[P]
SAVE DESCRIPTOR "voiture" END "file_1.txt" NEW CLOCK "vitesse" 2 "position" 3;
[/P]

{\em Idem}, mais seule la valeur finale des descripteurs sera sauvegardée ({\small {\tt END}} au lieu de {\small {\tt ALL}}).\\

[P]
SAVE DESCRIPTOR "voiture" END "file_1.txt" NEW CLOCK "vitesse" 2 "position" 3 (PARAM : ALL);
[/P]

{\em Idem}, mais l'entête du fichier comprendra les valeurs de tous les paramètres.\\

[P]
SAVE DESCRIPTOR "voiture" END "file_1.txt" NEW CLOCK "vitesse" 2 "position" 3 (PARAM : GROUP "VOITURE", GROUP "DRIVER");
[/P]

{\em Idem}, mais l'entête du fichier comprendra les valeurs de tous les paramètres dont le groupe est "VOITURE" ou bien "DRIVER".\\

[P]
SAVE DESCRIPTOR SIM_IDN "voiture" END "file_1.txt" NEW CLOCK "vitesse" 2 "position" 3 (PARAM : "carrefour" "nombre_entrees", "voiture" "intensiteArrivees");
[/P]

{\em Idem}, mais l'entête du fichier comprendra les valeurs de 2 paramètres particuliers. Chaque ligne débutera par le numéro de la simulation.\\

[P]
SAVE CREATION_DATE "voiture" "file_1.txt" APPEND DATE_MHJ;
[/P]

On demande que les dates de création de toutes les voitures soient rangées dans le fichier [F]file_1.txt[/F] en écrivant dans celui-ci après le dernier enregistrement (respect d'éventuelles données s'y trouvant déjà) ou à partir du début si le fichier est vide. Le fichier est créé s'il n'existe pas encore. La date est sous le format {\em minute heure jour\_julien}.\\

[P]
SAVE COMPONENTS "voiture" "file_1.txt" NEW DATE_HDM;
[/P]

On demande que les dates d'attribution des composants à toutes les voitures soient rangées dans le fichier [F]file_1.txt[/F]. La date est sous le format {\em heure jour\_du\_mois mois}.\\

[P]
SAVE ELEMENTS "carrefour" "file_1.txt" NEW DATE_DMY;
[/P]

On demande que les dates d'ajout d'éléments à tous les carrefours soient rangées dans le fichier [F]file_1.txt[/F]. La date est sous le format {\em jour\_du\_mois mois année}.\\
 */
//@{

//@} // fin Sauvegarde fichier séquentiel
/**@name Sauvegarde sur base de données
On détaille ici les principes de la sauvegarde de données dans une base relationnelle. La seule famille de bases gérée par la version actuelle de \Db est PostgreSQL.\\

Les spécifications ci-dessous requièrent que la base manipulée ait été préalablement créée dans le système hôte. Pour exemple, sous un système Linux, la base de nom 'ma_base' est créée (pour le compte de login 'durand') puis supprimée avec les commandes suivantes :
\begin{verbatim}
[P]createdb ma_base
...
dropdb ma_base[/P]
\end{verbatim}

Les spécifications autorisées :

[P]
DBEXPORT DESCRIPTOR "voiture" ALL "postgresql" "durand" "ma_base" CLOCK "vitesse" 2 "position" 3;
[/P]

[P]
DBEXPORT DESCRIPTOR SIM_IDN "voiture" ALL postgresql (durand ma_base v_speed) CLOCK "vitesse" 2 "position" 3;
[/P]

On demande que les valeurs de vitesse et de position soient sauvegardées dans une table de base de données relationnelle. La table réceptrice (de nom 'v_speed' dans l'exemple) doit avoir été créée (voir ci-dessous) avec un ensemble de champs englobant la liste des descripteurs mentionnés dans la spécification. Les spécifications CLOCK et ISO_8601 sont les seules autorisées pour le format de date.\\
Si la specification de la destination ne comprend pas le nom de la table (i.e. : '(durand ma_base)' seulement), alors les données seront envoyées dans une table de 'ma_base' dont le nom est celui de la classe d'entité concernée (ici 'voiture'). Bien entendu, cette table doit avoit été créée.\\
Le mot clé 'SIM_IDN' est optionnel. S'il est présent, chaque ligne envoyée dans la table contiendra une valeur significative pour le champ 'sim_idn'. Ce champ doit alors avoit été prévu dans la structure de la table. Si absent, l'éventuel champ 'sim_idn' de la table restera vide pour toutes les lignes envoyées. La valeur envoyée pour ce champ est le numéro de la simulation courante. Ce numéro, 1 par défaut, est généralement attribué par l'option {\small {\tt -idn}} de la ligne de commande (voir la page 'Interprétation des arguments de la commande de simulation'). Il est accessible dans le code utilisateur par le service [Y]GetNumId[/Y] de la classe [C]Simulation[/C].\\

[P]
DBNEWTABLE POSTGRESQL (durand ma_base v_speed) sim_idn integer instanceName text T integer position numeric vitesse numeric;
[/P]

On demande la création de la table 'v_speed' dans la base 'ma_base'. On précise l'utilisateur qui réalise cette tâche ('durand'). La base doit avoir été créée préalablement. Si la table existe déjà, cette demande est ignorée. La variante suivante est autorisée ... : \\

[P]
DBNEWTABLE POSTGRESQL (durand (mon_pwd) ma_base v_speed) sim_idn integer instanceName text T integer position numeric vitesse numeric;
[/P]

... qui permet de protéger l'accès à la table par un mot de passe.\\

La spécification de la base (trois ou quatre éléments) est suivie par la description des champs, chacun avec son nom et son type. Les noms des trois premiers champs de cet exemple sont spéciaux et réservés : le premier champ recevra pour valeur le numéro de la simulation, le deuxième recevra le nom de l'instance dont on sauvegarde l'état, le troisième recevra le moment auquel l'état sauvegardé se rapporte. Ces champs peuvent ne pas exister, et s'ils existent, ce peut être à un emplacement quelconque dans la table.\\
 */
//@{

//@} // fin Sauvegarde  base de données
/**@name Affichage

Les requêtes suivantes ne sont gérées que par la version de DIESE équipée pour l'utilisation interactive du simulateur (voir page 'Mise en Oeuvre de DIESE')\index{utilisation interactive}.\\

[P]
RECORD DESCRIPTOR "voiture" CLOCK "vitesse" 2 "position" 3;
[/P]

On ne demande pas que les valeurs de vitesse et de position soient sauvegardées sur un fichier, mais plutôt enregistrées en mémoire ({\small {\tt RECORD}}) et non affichées en temps réel à l'écran : le but est seulement de les visualiser graphiquement en fin de simulation. Les variables sont obligatoirement des descripteurs variables.

[P]
PLOT DESCRIPTOR "voiture" "vitesse" 2 LINE;
[/P]

On demande que les valeurs de la vitesse de toutes les voitures soient affichées à l'écran au cours de la simulation, sur des graphiques (autant que de voitures) où les points (x, y) seront reliés par des segments de droite. On ne garde que deux chiffres après la virgule. L'unique variable (ici la vitesse) est obligatoirement un descripteur variable. \\

[P]
PLOT DESCRIPTOR "voiture" "vitesse" 2 POINT;
[/P]

{\em Idem}, mais les points (x, y) ne seront pas reliés par des segments de droite.\\
 */
//@{

//@} // fin Affichage

//@} // fin Sauvegarde d'informations sur les entités

//@{

class ClockValueItem
//---------------------------------------------------------------------------- 
{
 public:
         // constructeur
  ClockValueItem(int, UnifiedValue);
  ClockValueItem(int, int, ValueTypeId);
  ClockValueItem(int, float, ValueTypeId);
  ClockValueItem(int, double, ValueTypeId);
  ClockValueItem(int, char*, ValueTypeId);
  ClockValueItem(int, Entity*, ValueTypeId);
         // destructeur
  ~ClockValueItem();
         // La date en unités de temps simulé.
  int           mClock;
         // La valeur associée à la date.
  UnifiedValue  mValue;
         // Le type de la valeur.
  ValueTypeId   mValueType;

         // Opérateur d'égalité. Renvoie {\em vrai} si la date et la valeur sont identiques.
  Boolean       operator==(const ClockValueItem&) const;
         // Opérateur d'inégalité. Renvoie {\em vrai} si la date ou la valeur sont différentes.
  Boolean       operator!=(const ClockValueItem&) const;

         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const ClockValueItem&) const {return TRUE;};
  Boolean         operator > (const ClockValueItem&) const {return TRUE;}; 
  
  // ci-dessous remplacement operator << par display (JPR 020715)
  // parce que la compilation avec Swig.o provoque une double définition
  // des opérateurs << de ostream et >> de istream
  //  friend ostream & operator << (ostream &, const ClockValueItem &);
  void          display() const;
         // Renvoie la date (en nombre d'unités de temps simulé depuis le début de la simulation) à laquelle se rapporte la valeur.
  int           GetClock() const;
         // Renvoie la valeur, qui doit être entière.\index{méthodes!GetIntValue - ClockValueItem}
  int           GetIntValue() const;
         // Renvoie la valeur, qui doit être flottante.\index{méthodes!GetFloatValue - ClockValueItem}
  float         GetFloatValue() const;
         // Renvoie la valeur, qui doit être en double précision.\index{méthodes!GetDoubleValue - ClockValueItem}
  double        GetDoubleValue() const;
         // Renvoie la valeur, qui doit être une chaîne de caractères (moins de 50 caractères).\index{méthodes!GetStringValue - ClockValueItem}
  char*         GetStringValue() const;
         // Renvoie la valeur, qui doit être un pointeur sur une entité.\index{méthodes!GetEntityValue - ClockValueItem}
  BasicEntity* GetEntityValue() const;
         // Renvoie la valeur.\index{méthodes!GetValue - ClockValueItem}
  UnifiedValue  GetValue() const;
         // Renvoie le type de la valeur.\index{méthodes!GetValueType - ClockValueItem}
ValueTypeId     GetValueType() const;
};

void UnifiedValueToString(ClockValueItem*, int, char [50]);
void UnifiedValueToString(ClockValueItem*, char [50]);

class DescTimeSeries
//---------------------------------------------------------------------------- 
{
public:
  DescTimeSeries(DescriptorOS*);
  ~DescTimeSeries();

         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const DescTimeSeries&) const {return TRUE;};
  Boolean         operator > (const DescTimeSeries&) const {return TRUE;};

  DescriptorOS*        GetDescriptorOS() const;
                       
  void                 AddClockIntValueItem(int, int);
  void                 AddClockFloatValueItem(int, float);
  void                 AddClockDoubleValueItem(int, double);
  void                 AddClockStringValueItem(int, char*);
  
  void                 GoHead();
  void                 MyDisplay();  // to scratch
  void                 GoNext();
  bool                 GoNext2(bool msg);  // + JPR 090414 modifié 100212
  void                 GoTail();
  ClockValueItem*      GetCurrent();
  Boolean              IsCurrentIn();

protected:
  DescriptorOS*        mpDescriptorOS;
  pClockValueList      mpClockValueList;
};

class EntityOutputSpec;

class EntityInstanceOS
//---------------------------------------------------------------------------- 
{
public:
  EntityInstanceOS(int, EntityOutputSpec*);
  ~EntityInstanceOS();

  char*                GetEntityInstanceName() const ;
  int                  GetCreationClock() const;
  int                  GetDeletionClock() const;
  pChangeStructureList GetChangeComponentList();
  pChangeStructureList GetChangeElementList();
  pDescTimeSeriesTab   GetDescTimeSeriesTab() const;
  EntityOutputSpec*    GetEntityOutputSpec() const;
  bool                 ExistsDTS(char*);

  void                 SetDeletionClock(int);
  void                 SetEntityInstanceName(char*);
  void                 AddChangeComponentData(ChangeStructureItem*);
  void                 AddChangeElementData(ChangeStructureItem*);
  void                 AddDescTimeSeries(DescTimeSeries*);

protected:
  char*                mEntityInstanceName;
  int                  mCreationClock;
  int                  mDeletionClock;
  pChangeStructureList mpChangeComponentList;
  pChangeStructureList mpChangeElementList;
  pDescTimeSeriesTab   mpDescTimeSeriesTab;
  EntityOutputSpec*    mpEntityOutputSpec;
};

class MultipleDescriptorOS
//---------------------------------------------------------------------------- 
{
public:
  MultipleDescriptorOS(TimeType, Boolean, Boolean, Boolean, AllOrEnd);
  MultipleDescriptorOS(Boolean, Boolean, Boolean);
  ~MultipleDescriptorOS();

  int                  mSubClass; // 1=OutputFile  2=Database // + JPR 120316
  int                  GetSubClass();

  void                 SetTimeType(TimeType);
  TimeType             GetTimeType() const;
  void                 SetAllOrFinal(AllOrEnd);
  AllOrEnd             GetAllOrFinal() const;
  pDescriptorOSList    GetDescriptorsData();
  
  Boolean              IsPlotted() const;
  Boolean              IsRecorded() const;
  Boolean              IsSaved() const;

  EntityOutputSpec*    GetEntityOutputSpec() const;
  
  void                 SetEntityOutputSpec(EntityOutputSpec*);
  void                 AddDescriptorOS(char*, int);

  virtual void         DisplayTimeSeries()=0;
  void                 RegisterEntityOS(char* className);

  void SetIsNumIdentified(Boolean); // JPR 080204
  Boolean GetIsNumIdentified() const;

protected:
  Boolean              mIsNumIdentified; // JPR 080204
  TimeType             mTimeType;
  AllOrEnd             mAllOrFinal; 

  Boolean              mIsPlotted;
  Boolean              mIsRecorded;
  Boolean              mIsSaved; 

  pDescriptorOSList    mDescriptorsData;
  EntityOutputSpec*    mpEntityOutputSpec;
};

class OutputFileMDescOS: public MultipleDescriptorOS
//---------------------------------------------------------------------------- 
{
public:
  OutputFileMDescOS(TimeType,Boolean, Boolean, Boolean, GraphType, AllOrEnd);
  ~OutputFileMDescOS();
  
  OutputFile*          GetOutputFile() const;
  GraphType            GetGraphType() const;

  pParameterOSList     GetParametersData();
  void                 AddParameterOS(char*, char*);

  void ConfigureOutputFile(char* fileName, OutputMode om);

  void DisplayTimeSeries();

protected:
  OutputFile*          mpOutputFile;
  GraphType            mGraphType;
  pParameterOSList     mParametersData; // + JPR 061024
};

class DatabaseMDescOS: public MultipleDescriptorOS
//---------------------------------------------------------------------------- 
{
public:
  DatabaseMDescOS(Boolean, Boolean, Boolean);
  ~DatabaseMDescOS();

  void ConfigureDatabase(const char* dbUserName, const char* dbName, const char* dbTableName, const char* dbPassword="");

  void DisplayTimeSeries();

  virtual void BeginSQLInsertion( std::vector<std::string> colNames) = 0;
  virtual void FinishSQLInsertion() = 0;

  virtual void NextValue(std::string val) = 0;
  virtual void BeginRecord() = 0;
  virtual void EndRecord() = 0;

  virtual std::string FormatTextValue(const std::string) = 0;

protected:
  std::string mDbUserName;
  std::string mDbPassword;
  std::string mDbName;
  std::string mDbTableName;
};

#ifdef _POSTGRESQL_
class PostgresqlMDescOS: public DatabaseMDescOS
//---------------------------------------------------------------------------- 
{
public:
  PostgresqlMDescOS(Boolean, Boolean, Boolean);
  ~PostgresqlMDescOS();

  void BeginSQLInsertion(std::vector<std::string> colNames);
  void FinishSQLInsertion();

  void NextValue(std::string val);
  void BeginRecord();
  void EndRecord();

  std::string FormatTextValue(const std::string val)  {return val;};

protected:
  PGconn* mDbConnection;
  bool mIsReadyForCopy;
  std::string mCopyBuffer;
};

class AltPostgresqlMDescOS: public DatabaseMDescOS
//---------------------------------------------------------------------------- 
{
public:
  AltPostgresqlMDescOS(Boolean, Boolean, Boolean);
  ~AltPostgresqlMDescOS();

  void BeginSQLInsertion(std::vector<std::string> colNames);
  void FinishSQLInsertion();

  void NextValue(std::string val);
  void BeginRecord();
  void EndRecord();

  std::string FormatTextValue(const std::string val)  {return (std::string) ("\'" + val + "\'");};

  void ProcessSQLQuery();

protected:
  PGconn* mDbConnection;
  std::string mQuery;

  bool mFirstValue;
  bool mFirstRecord;
};

#endif

class MDescOSParticipation
//---------------------------------------------------------------------------- 
{
public:
  MDescOSParticipation(MultipleDescriptorOS*, int);
  ~MDescOSParticipation();

  MultipleDescriptorOS* GetMDescOS() const;
  int                   GetPrecisionInMDescOS() const;

  MultipleDescriptorOS* mpMDescOS;
  int                   mPrecisionInMDescOS;
};

class DescriptorOS
//---------------------------------------------------------------------------- 
{
public:
  DescriptorOS(char*);
  ~DescriptorOS();
  
  char*                  GetDescName() const;
  UserDescriptorId       GetDescId() const;
  ValueTypeId            GetDescType() const;
  pMDescOSParticipationList GetMDescOSParticipationList();

  void                   SetDescType(ValueTypeId);
  void                   AddMDescOSParticipation(MDescOSParticipation*);
   
protected:
  char*                  mDescName;
  UserDescriptorId       mDescId;             // vérifier que c'est utile
  ValueTypeId            mDescType;
  pMDescOSParticipationList mMDescOSParticipationList;
};

class ParameterOS
//---------------------------------------------------------------------------- 
{
public:
  ParameterOS(char*, char* /*, ValueTypeId, UnifiedValue*/);
  ~ParameterOS();
  
  char*                  GetIdent1() const;
  char*                  GetIdent2() const;
  //ValueTypeId            GetValueTypeId() const;
  //UnifiedValue           GetUnifiedValue() const;
   
protected:
  char*                  mIdent1;
  char*                  mIdent2;
  //ValueTypeId            mParamType;
  //UnifiedValue           mValue;
};
  

class EntityOutputSpec : public OutputSpec
//---------------------------------------------------------------------------- 
{
public:

// LIFECYCLE

          // Constructeur complet. Les arguments sont dans l'ordre, le nom de la classe d'entités, le mode d'impression du temps, trois booléens indiquant respectivement si la classe fait l'entité d'une sauvegarde sur fichier, d'un enregistrement en mémoire seulement ou d'une sortie graphique interactive, et les trois fichiers où pourront être sauvegardés les historiques sur les composants, les éléments et les valeurs des descripteurs.
   EntityOutputSpec(char*, TimeType, Boolean, Boolean, Boolean,
		OutputFile*, OutputFile*, OutputFile*); 
          // Destructeur.                
   ~EntityOutputSpec(); 

// ACCES

          /// Renvoie le symbole de la classe dont les instances sont l'objet 'entité' de la spécification de sortie.
UserClassId             TargetClassId() const;

          // Etablit si les informations sur la création de l'entité doivent être enregistrées sur fichier.\index{méthodes!IsCreationSaved}
void                  IsCreationSaved(Boolean);
          // Renvoie {\em vrai} si les informations sur la création de l'entité doivent être enregistrées sur fichier.
Boolean               IsCreationSaved() const;

          // Etablit si les informations sur les ajouts et retraits de composants doivent être enregistrées sur fichier.\index{méthodes!IsChangeComponentSaved}
void                  IsChangeComponentSaved(Boolean);
          // Renvoie {\em vrai} si les informations sur les ajouts et retraits de composants doivent être enregistrées sur fichier.
Boolean               IsChangeComponentSaved() const;

          // Etablit si les informations sur les ajouts et retraits d'éléments doivent être enregistrées sur fichier.\index{méthodes!IsChangeElementSaved}
void                  IsChangeElementSaved(Boolean);
          // Renvoie {\em vrai} si les informations sur les ajouts et retraits d'éléments doivent être enregistrées sur fichier.
Boolean               IsChangeElementSaved() const;

          // Affecte un fichier sur lequel les informations sur la création de l'entité doivent être enregistrées.\index{méthodes!SetCreationOutputFile}
void                  SetCreationOutputFile(OutputFile*);
          // Renvoie le fichier sur lequel les informations sur la création de l'entité doivent être enregistrées.\index{méthodes!GetCreationOutputFile}
OutputFile*           GetCreationOutputFile() const;

          // Affecte un fichier sur lequel les informations sur les ajouts et retraits de composants doivent être enregistrées.\index{méthodes!SetChangeComponentOutputFile}
void                  SetChangeComponentOutputFile(OutputFile*);
          // Renvoie le fichier sur lequel les informations sur les ajouts et retraits de composants doivent être enregistrées.\index{méthodes!GetChangeComponentOutputFile}
OutputFile*           GetChangeComponentOutputFile() const;

          // Affecte un fichier sur lequel les informations sur les ajouts et retraits de éléments doivent être enregistrées.\index{méthodes!SetChangeElementOutputFile}
void                  SetChangeElementOutputFile(OutputFile*);
          // Renvoie le fichier sur lequel les informations sur les ajouts et retraits de éléments doivent être enregistrées.\index{méthodes!GetChangeElementOutputFile}
OutputFile*           GetChangeElementOutputFile() const;

          // Renvoie une liste, dont chaque élément contient les informations sur une instance de la classe d'entités en question.\index{méthodes!GetInstancesData - EntityOutputSpec}
pEntityInstanceOSList     GetInstancesData();

          // Ajoute un élément contenant les informations sur une instance de la classe d'entités en question.\index{méthodes!AddInstanceData - EntityOutputSpec}
void                  AddInstanceData(EntityInstanceOS*);

          // Enlève un élément contenant les informations sur une instance de la classe d'entités en question.\index{méthodes!RemoveInstanceData - EntityOutputSpec}
//EntityInstanceOS*         RemoveInstanceData(EntityInstanceOS*);
 bool         RemoveInstanceData(EntityInstanceOS*); // gcc 4.1

          // Renvoie une liste, dont chaque élément permet de retrouver toutes les requêtes conjointes sur une classe de descripteurs de la classe d'entités en question.\index{méthodes!GetDescriptorsData - EntityOutputSpec}
pDescriptorOSList             GetDescriptorsData();
  
          // Ajoute un élément contenant une requête sur une classe de descripteurs de la classe d'entités en question, si un tel élément n'existe pas déjà pour cette classe de descripteurs.\index{méthodes!AddDescriptorOS - EntityOutputSpec}
  void                        AddDescriptorOSIfNew(DescriptorOS*);

          // Renvoie une liste, dont chaque élément contient une requête conjointe sur plusieurs classes de descripteurs de la classe d'entités en question.\index{méthodes!GetMultipleDescriptorsData - EntityOutputSpec}
pMultipleDescriptorOSList     GetMultipleDescriptorsData();
  
          // Ajoute un élément contenant une requête conjointe sur plusieurs classes de descripteurs de la classe d'entités en question.\index{méthodes!AddMultipleDescriptorOS - EntityOutputSpec}
  void                        AddMultipleDescriptorOS(MultipleDescriptorOS*); 
          // Opère le transfert sur fichier des informations sauvegardées au titre d'une classe d'entités.\index{méthodes!DisplayOneOS}
   void                    DisplayChangesOneOS();
   void                    DisplayStructureChangesOneOS();
   void                    DisplayDescValueChangesOneOS();  

          // Opère le transfert sur fichier des informations sauvegardées au titre des classes d'entités.\index{méthodes!DisplayAllOS}

   static  void             DisplayAllOS(UserClassId id);  

          // Renvoie la spécification concernant la classe dont le nom est passé en argument, et qui mentionne la sauvegarde de la date de création des instances.\index{méthodes!GetCreationDateOS}
   static EntityOutputSpec*    GetCreationDateOS(char*);

   static pOutputSpecList  InstanceList();

   static void  EraseInstanceList();  // + JPR 120308

   static void  PushBackInstanceList(EntityOutputSpec*);  // + JPR 120308

          // Renvoie vrai s'il existe des spécifications de sortie pour les entités.\index{méthodes!AreOutputSpec}
   static Boolean          AreOutputSpec(); 

   static void  DisplayInstanceList();  // + JPR 120308  

   static void  DisplayInstanceListAddress();  // + JPR 120308


// OPERATIONS

          // Ajoute l'information de l'occurrence d'un événement de nom de classe {\small {\tt mpTargetClassName}}. Cette information est le couple formé du nom de classe de l'événement et de la date d'occurrence, passés en arguments.\index{méthodes!AddOccurrenceData - EntityOutputSpec}
void                AddOccurrenceData(char*, int);


// INQUIRIES

protected:   

   static pOutputSpecList  mpInstanceList; 
   UserClassId             mTargetClassId;  // + 061026
         
   Boolean               mIsCreationSaved;
   Boolean               mIsChangeComponentSaved;
   Boolean               mIsChangeElementSaved;

   OutputFile*           mpCreationOutputFile;
   OutputFile*           mpChangeComponentOutputFile;
   OutputFile*           mpChangeElementOutputFile;

   pEntityInstanceOSList     mInstancesData;    
   pDescriptorOSList         mDescriptorsData;
   pMultipleDescriptorOSList mMultipleDescriptorsData;

};

//@}  // fin sauvegarde Entity

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Sauvegarde de la chronique des événements 
 L'utilisateur peut demander la sauvegarde sur un ou plusieurs fichiers des informations sur les dépilements d'événements survenus au cours de la simulation. Une telle demande porte soit sur l'ensemble des classes d'événements (les informations seront alors sauvegardées sur le même fichier), soit sur une classe d'événements particulière.\label{defStructEventData} \\

Voici des exemples des diverses requêtes admises :\\

[P]
SAVE EVENT "classe\_event\_1" "file_1.txt" NEW DATE\_MHY;
[/P]

Les dépilements des événements de nom de classe {\small {\tt "classe\_event\_1"}} seront sauvegardés dans le fichier [F]file_1.txt[/F] en écrivant dans celui-ci à partir de son début ({\small {\tt NEW}} : écrasement d'éventuelles données s'y trouvant déjà). Le fichier est créé s'il n'existe pas encore.\\

[P]
SAVE EVENT "classe\_event\_1" "file_1.txt" APPEND DATE\_HDM;
[/P]

Les dépilements des événements de nom de classe {\small {\tt "classe\_event\_1"}} seront sauvegardés dans le fichier [F]file_1.txt[/F] en écrivant dans celui-ci après le dernier enregistrement ({\small {\tt APPEND}} : respect d'éventuelles données s'y trouvant déjà) ou à partir du début si le fichier est vide. Le fichier est créé s'il n'existe pas encore.\\

[P]
SAVE EVENT "all\_eventClasses" "file_2.txt" NEW DATE\_DMY;
[/P]

Les dépilements des événements de toutes les classes seront sauvegardés dans le fichier [F]file_2.txt[/F]. Une telle requête inhibe toutes les requêtes portant sur une classe particulière, qu'elles soient déclarées antérieurement ou postérieurement dans le fichier de spécification des sorties.\\

Chaque demande se traduit par la création d'une instance de la classe [C]EventOutputSpec[/C]. Juste après cette création, une telle instance ne contient que le nom de la classe d'événements (ou la chaîne de caractères "all\_eventClasses") et la référence à un fichier externe sur lequel on veut sauvegarder les informations. Au cours de la simulation, et plus précisément lors de chaque dépilement d'un événement de l'agenda, un couple \{{\em nom de classe de l'événement}, {\em date de dépilement}} est ajouté à une liste attachée à l'instance de [C]EventOutputSpec[/C] appropriée. \\
La liaison entre une classe d'événements et l'instance de [C]EventOutputSpec[/C] correspondante est faite lors de l'attribution d'un nom de classe (par [Y]ClassName[/Y] exclusivement) aux événements de la classe, généralement dans le constructeur de la classe (voir page 'La classe des événements'). \Db cherche à ce moment s'il a été créé une instance de [C]EventOutputSpec[/C] dont la chaîne de caractères renvoyée par [Y]TargetClassName[/Y] est identique au nom de classe de l'événement en cours de construction. En cas de succès, la référence à l'instance de [C]EventOutputSpec[/C] est attachée à l'événement, pour être exploitée lors de son dépilement. \\
La fonction [Y]DisplayEventOutputSpecs[/Y] fournit une exploitation standard des enregistrements des dépilements d'événements.
*/
//@{
         // Les couples {nom de classe, date}.

//La chronique des dépilements d'événements est enregistrée dans une liste, attribut {\small {\tt mDataList}} de l'instance de [C]EventOutputSpec[/C]. Chaque élément de cette liste est un pointeur sur une instance de la présente classe. C'est un couple \{{\em nom de classe de l'événement}, {\em date de dépilement}}.


class StructEventData
{
 public:
         // constructeur
  StructEventData(char*, int);
         // destructeur
  ~StructEventData();

// OPERATORS   
         // Renvoie le nom de classe de l'événement.
  char*         GetClassName() const;
         // Renvoie la date de dépilement (en nombre d'unités de temps simulé depuis le début de la simulation).
  int           GetPopClock() const;

protected:
         // Le nom de classe de l'événement. La valeur est soit la valeur renvoyée par un message [Y]ClassName[/Y]adressé à l'événement lorsque la spécification de sortie comporte ALL au lieu d'un nom de classe, soit celle renvoyée par [Y]TargetClassName[/Y] lorsque la spécification de sortie comporte un nom de classe.
  char*         mClassName;
         // La date de dépilement.
  int           mPopClock;
};

          // classe des spécifications de sortie pour les événements
  
class EventOutputSpec : public OutputSpec
{
public:

// LIFECYCLE

          // Constructeur. L'argument est le nom de la classe d'événements dont les instances sont l'entité de la spécification de sortie. Cela peut être la chaîne de caractères "all_eventClasses". Dans ce cas, toutes les classes d'événements sont les entités de la spécification de sortie.
   EventOutputSpec(char*, TimeType, OutputFile*); 
          // Destructeur.                
   ~EventOutputSpec(); 

// ACCESS

          // Etablit si les informations sur l'événement doivent être enregistrées sur fichier.\index{méthodes!IsSaved}
void                  IsSaved(Boolean);
          // Renvoie {\em vrai} si les informations sur l'événement doivent être enregistrées sur fichier.
Boolean               IsSaved() const;
          // Affecte un fichier sur lequel les informations sur l'événement doivent être enregistrées.\index{méthodes!SetOutputFile}
void                  SetOutputFile(OutputFile*);
          // Renvoie le fichier sur lequel les informations sur l'événement doivent être enregistrées.\index{méthodes!GetOutputFile}
OutputFile*           GetOutputFile() const;

          // Opère le transfert sur fichier des informations sauvegardées au titre d'une classe d'événement.\index{méthodes!DisplayOneOS}
   void                    DisplayOneOS();  

          // Opère le transfert sur fichier des informations sauvegardées au titre des classes d'événements.\index{méthodes!DisplayAllOS}
   static void             DisplayAllOS();    

   static pOutputSpecList  InstanceList();

   static void  EraseInstanceList();  // + JPR 120308 

   static void  PushBackInstanceList(EventOutputSpec*);  // + JPR 120308

   static void  RemoveFirstInstanceList(EventOutputSpec*);  // + JPR 120308

          // Renvoie vrai s'il existe des spécifications de sortie pour les événements.\index{méthodes!AreOutputSpec}
   static Boolean          AreOutputSpec();  

// OPERATIONS
 
          // Ajoute l'information de l'occurrence d'un événement de nom de classe [Y]mpTargetClassName[/Y]. Cette information est le couple formé du nom de classe de l'événement et de la date d'occurrence, passés en arguments.\index{méthodes!AddOccurrenceData - EventOutputSpec}
void                AddOccurrenceData(char*, int);

// INQUIRIES

protected:   
   static pOutputSpecList  mpInstanceList; 
         
   Boolean               mIsSaved;
   OutputFile*           mpOutputFile;
   pStructEventDataList  mDataList;

};
//@}    // fin Sauvegarde événements

//-X///////////////X/////////////////////////////X////////////////////////////


/**@name Sauvegarde de la chronique des processus 
L'utilisateur peut demander la sauvegarde sur un ou plusieurs fichiers des informations sur la vie des processus au cours de la simulation. Une telle demande porte soit sur l'ensemble des classes de processus (les informations sur les différentes classes seront alors sauvegardées sur le même fichier), soit sur une classe particulière de processus. \\

Voici des exemples des diverses requêtes admises :\\

[P]
SAVE PROCESS "classe\_process\_1" "file_1.txt" NEW DATE\_MHJ;
[/P]

Les informations sur les processus de nom de classe {\small {\tt "classe\_process\_1"}} seront sauvegardées dans le fichier [F]file_1.txt[/F] en écrivant dans celui-ci à partir de son début ({\small {\tt NEW}} : écrasement d'éventuelles données s'y trouvant déjà). Le fichier est créé s'il n'existe pas encore.\\

[P]
SAVE PROCESS "classe\_process\_1" "file_1.txt" APPEND DATE\_HDM;
[/P]

Les informations sur les processus de nom de classe {\small {\tt "classe\_process\_1"}} seront sauvegardées dans le fichier [F]file_1.txt[/F] en écrivant dans celui-ci après le dernier enregistrement ({\small {\tt APPEND}} : respect d'éventuelles données s'y trouvant déjà) ou à partir du début si le fichier est vide. Le fichier est créé s'il n'existe pas encore.\\

[P]
SAVE PROCESS "all\_processClasses" "file_2.txt" NEW DATE\_DMY;
[/P]

Les informations sur les processus de toutes les classes seront sauvegardées dans le fichier [F]file_2.txt[/F]. Une telle requête inhibe toutes les requêtes portant sur une classe particulière, qu'elles soient déclarées antérieurement ou postérieurement dans le fichier de spécification des sorties.\\

Chaque demande se traduit par la création d'une instance de [C]ProcessOutputSpec[/C]. Juste après cette création, une telle instance ne contient que le nom de la classe de processus (ou la chaîne de caractères "all\_processClasses") et la référence à un fichier externe sur lequel on veut sauvegarder les informations. Au cours de la simulation, des quintuplets \{{\em type de processus}, {\em nom de classe de processus}, {\em date d'initialisation ou d'exécution}, {\em date d'arrêt}, {\em entité concernée}} sont ajoutés à une liste attachée à l'instance de [C]ProcessOutputSpec[/C] appropriée. \\
La liaison entre une classe de processus et l'instance de [C]ProcessOutputSpec[/C] correspondante est faite lors de l'attribution d'un nom de classe (par [Y]ClassName[/Y] exclusivement) aux processus de la classe, généralement dans son constructeur (voir pages 'La classe des processus ponctuels' et 'La classe des processus continus'). \Db cherche alors s'il a été créé une instance de [C]ProcessOutputSpec[/C] dont la chaîne de caractères passée en argument du constructeur et renvoyée par [Y]TargetClassName[/Y] est identique au nom de classe du processus en cours de construction. En cas de succès, la référence à l'instance de [C]ProcessOutputSpec[/C] est attachée au processus, pour être exploitée lors de son initialisation, de son arrêt ou de son exécution.\\
Les fonctions [Y]DisplayDProcessOutputSpecs[/Y], [Y]DisplayCProcessOutputSpecs[/Y] et [Y]DisplayProcessOutputSpecs[/Y]  fournissent une exploitation standard des enregistrements des actions sur les processus, respectivement ponctuels, continus et sans distinction de type. 
*/
//@{

         // Les quintuplets {type, nom de classe, date_1, date_2, entité}.  
//La chronique des processus (initialisation, arrêt, exécution) sur les processus est enregistrée dans une liste, attribut {\small {\tt mDataList}}. Chaque élément de cette liste est un pointeur sur une instance de la présente classe. C'est un quintuplet {{\em type de processus}, {\em nom de classe}, {\em date d'initialisation/exécution}, {\em date d'arrêt}, {\em entité concernée}}

class StructProcessData
{
 public:
         // constructeur
  StructProcessData(ProcessSubclassId, char*, int, 
				       int, int, char*);
         // destructeur
  ~StructProcessData();

// OPERATORS   

         // Renvoie le type de classe du processus.
  ProcessSubclassId   GetClassType() const;
         // Renvoie le nom de classe du processus.
  char*            GetClassName() const;
         // Renvoie le numéro d'instance du processus.
  int              GetInstanceNumber() const;
         // Renvoie la date d'initialisation (pour un processus continu) ou d'exécution (pour un processus ponctuel), en nombre d'unités de temps simulé depuis le début de la simulation.
  int              GetInitOrExecClock() const;
         // Renvoie la date d'arrêt (pour un processus continu), en nombre d'unités de temps simulé depuis le début de la simulation.
  int              GetStopOrVoidClock() const;
         // Etablit la date d'arrêt (pour un processus continu), en nombre d'unités de temps simulé depuis le début de la simulation.
  void             SetStopOrVoidClock(int);
         // Renvoie le nom d'instance de l'entité à laquelle se rapporte le processus.
  char*            GetProcessedEntityInstanceName() const;

protected:   
         // Le type du processus : continu ou ponctuel.
  ProcessSubclassId  mClassType;
         // Le nom de classe du processus.
  char*           mClassName;
         // Le numéro d'instance du processus.
  int             mInstanceNumber;
         // La date d'initialisation (pour un processus continu) ou d'exécution (pour un processus ponctuel), en unités de temps simulé.
  int             mInitOrExecClock;
         // La date d'arrêt (pour un processus continu), en unités de temps simulé.
  int             mStopOrVoidClock;
         // Le nom d'instance de l'entité à laquelle se rapporte le processus.
  char*           mProcessedEntityInstanceName;
};

          // classe des spécifications de sortie pour les processus en général
  
class ProcessOutputSpec : public OutputSpec
{
public:

// LIFECYCLE

          // Constructeur. Le premier argument est un des deux symboles suivants de l'énumération OutputSpecType : CP_OUTPUTSPEC et DP_OUTPUTSPEC. Le second argument est le nom de la classe de processus dont les instances sont l'entité de la spécification de sortie.
   ProcessOutputSpec(OutputSpecType, char*, TimeType, OutputFile*);
          // Constructeur. Le seul argument doit être la chaîne de caractères "all_processClasses". Dans ce cas, toutes les classes de processus sont les entités de la spécification de sortie.  
   ProcessOutputSpec(char*, TimeType, OutputFile*); 
          // Destructeur.                
   ~ProcessOutputSpec(); 


// ACCESS

          // Etablit si les informations sur le processus doivent être enregistrées sur fichier.\index{méthodes!IsSaved}
void                IsSaved(Boolean);
          // Renvoie {\em vrai} si les informations sur le processus doivent être enregistrées sur fichier.
Boolean             IsSaved() const;
          // Affecte un fichier sur lequel les informations sur le processus doivent être enregistrées.\index{méthodes!SetOutputFile}
void                SetOutputFile(OutputFile*);
          // Renvoie le fichier sur lequel les informations sur le processus doivent être enregistrées.\index{méthodes!GetOutputFile}
OutputFile*         GetOutputFile() const;

          // Renvoie {\em vrai} si la spécification porte sur une classe de processus ponctuels.\index{méthodes!IsDPOutputSpec}
Boolean             IsDPOutputSpec() const;
          // Renvoie {\em vrai} si la spécification porte sur une classe de processus continus.\index{méthodes!IsDPOutputSpec}
Boolean             IsCPOutputSpec() const;
          // Renvoie {\em vrai} si la spécification porte sur toutes les classe de processus.\index{méthodes!IsPOutputSpec}
Boolean             IsPOutputSpec() const; 

          // Opère le transfert sur fichier des informations sauvegardées au titre d'une classe de processus.\index{méthodes!DisplayOneOS}
   void                    DisplayOneCPOS();   
   void                    DisplayOneDPOS();   
   void                    DisplayOneOS();   

          // Opère le transfert sur fichier des informations sauvegardées au titre des classes de processus.\index{méthodes!DisplayAllOS}
   static void             DisplayAllCPOS();  
   static void             DisplayAllDPOS();
   static void             DisplayAllOS();    

   static pOutputSpecList  InstanceList(); 

   static void  EraseInstanceList();  // + JPR 120308 

   static void  PushBackInstanceList(ProcessOutputSpec*);  // + JPR 120308 

   static void  RemoveFirstInstanceList(ProcessOutputSpec*);  // + JPR 120308

          // Renvoie vrai s'il existe des spécifications de sortie pour les processus.\index{méthodes!AreOutputSpec}
   static Boolean          AreOutputSpec();  

// OPERATIONS

          // Ajoute l'information de l'initialisation d'un processus continu ou d'exécution d'un processus ponctuel, de nom de classe {\small {\tt mpTargetClassName}}. Cette information est le quintuplet formé du nom de classe du processus, de la date d'initialisation ou d'exécution, du numéro d'instance attribué au processus, du nom d'instance de l'entité du processus, passés en arguments, et d'une date d'arrêt non significative.\index{méthodes!AddOccurrenceData - ProcessOutputSpec}
void                AddOccurrenceData(ProcessSubclassId, char*, int, int, char*);
 
          // Ajoute l'information de l'arrêt d'un processus de nom de classe {\small {\tt mpTargetClassName}}. Cet appel complémente le quintuplet formé au moment de l'initialisation d'un processus continu avec la date d'arrêt effective. La premier argument est le numéro d'instance du processus, passé pour permettre la recherche de l'information d'initialisation dans la liste {\small {\tt mDataList}}.
void                AddOccurrenceData(int, int); 

// INQUIRIES

protected:   
   static pOutputSpecList  mpInstanceList;   
         
   Boolean           mIsSaved;
   OutputFile*       mpOutputFile;  
         
   pStructProcessDataList  mDataList;

};

//@}  // fin sauvegarde processus

//-X///////////////X/////////////////////////////X////////////////////////////


// INLINE METHODS 
                         
// ACCESS


// EXTERNAL REFERENCES

// ---------------
// EXTERNAL FUNCTIONS
// ---------------
/**@name Fonctions 
\\
{\large {\tt void DeleteOutputSpecs()}} : \\

\DTab4 Récupère la mémoire allouée dynamiquement au titre de toutes les opérations de sauvegarde. Attention ! Cette fonction ne doit pas être invoquée avant de désallouer les entités auxquels les présentes spécifications font référence. Par exemple typique, la séquence ...
\begin{verbatim}
[P]DeleteOutputSpecs();
DeleteEntityInstances();[/P]
\end{verbatim}
...provoque une erreur de segmentation.\index{fonctions!DeleteOutputSpecs} 

{\large {\tt void DisplayEntityOutputSpecs(UserClassId)}} : \\

\DTab4 Opère le transfert sur fichier des informations sauvegardées en mémoire au titre de la classe d'entités dont le symbole de classe est passé en argument, ou de toutes les classes d'entités ayant fait l'entité d'une spécification si l'argument est BD_ENTITY.\index{fonctions!DisplayEntityOutputSpecs}\label{defDisplayOS}
 
{\large {\tt void DisplayEventOutputSpecs()}} : \\

\DTab4 Opère le transfert sur fichier des informations sauvegardées en mémoire au titre des classes d'événements.\index{fonctions!DisplayEventOutputSpecs}
 
{\large {\tt void DisplayCProcessOutputSpecs()}} : \\

\DTab4 Opère le transfert sur fichier des informations sauvegardées en mémoire au titre des classes de processus continus.\index{fonctions!DisplayCProcessOutputSpecs}
 
{\large {\tt void DisplayDProcessOutputSpecs()}} : \\

\DTab4 Opère le transfert sur fichier des informations sauvegardées en mémoire au titre des classes de processus ponctuels.\index{fonctions!DisplayCDrocessOutputSpecs}
 
{\large {\tt void DisplayProcessOutputSpecs()}} : \\

\DTab4 Opère le transfert sur fichier des informations sauvegardées en mémoire au titre des classes de processus continus et ponctuels.\index{fonctions!DisplayProcessOutputSpecs}
 
{\large {\tt void DisplayAllOutputSpecs()}} : \\

\DTab4 Opère le transfert sur fichier des informations sauvegardées en mémoire au titre des classes d'entités, de processus et d'événements.\index{fonctions!DisplayAllOutputSpecs}

*/
//@{
void DeleteOutputSpecs();

void DisplayEntityOutputSpecs(UserClassId);

void DisplayEventOutputSpecs();

void DisplayCProcessOutputSpecs();

void DisplayDProcessOutputSpecs();

void DisplayProcessOutputSpecs();

void DisplayAllOutputSpecs();


	// Construction d'une structure de sauvegarde d'informations historiques sur la structure des entités.
EntityOutputSpec*         SetEntityStructureOS(char*, 
				       TimeType,  
				       Boolean, Boolean, Boolean,
				       char*, char*, char*, 
				       OutputMode);
	// Construction d'une structure de sauvegarde d'informations historiques sur la valeur des descripteurs des entités.
MultipleDescriptorOS* SetMultipleDescriptorOS(char*, 
				      TimeType, 
				      Boolean bp, Boolean br, Boolean bs,
				      GraphType, AllOrEnd,
				      char*, OutputMode);
	// Construction d'une structure de sauvegarde d'informations historiques sur les événements.
EventOutputSpec*      SetEventOutputSpec(char*, TimeType,  
				    char*, OutputMode);
	// Construction d'une structure de sauvegarde d'informations historiques sur les processus d'une classe particulière. Le premier argument est DP_OUTPUTSPEC ou CP_OUTPUTSPEC.
ProcessOutputSpec*    SetProcessOutputSpec(OutputSpecType, char*, TimeType, 
					char*, OutputMode);
	// Construction d'une structure de sauvegarde d'informations historiques sur les processus de toutes les classes.
ProcessOutputSpec*    SetProcessOutputSpec(char*, TimeType, 
					char*, OutputMode);

//@}; // fin fonctions

//@} // fin spécifications de sortie

#endif  // _OUTPUTSPEC_HH
